@@ -3,9 +3,11 @@ const AppletManager = imports.ui.appletManager;
 const Clutter = imports.gi.Clutter;
 const St = imports.gi.St;
 const Cinnamon = imports.gi.Cinnamon;
+const CMenu = imports.gi.CMenu;
 const Lang = imports.lang;
 const Gio = imports.gi.Gio;
 const PopupMenu = imports.ui.popupMenu;
+const Main = imports.ui.main;
 const GLib = imports.gi.GLib;
 const Tooltips = imports.ui.tooltips;
 const DND = imports.ui.dnd;
@@ -43,6 +45,12 @@ class PanelAppLauncherMenu extends Applet.AppletPopupMenu {
         this._signals.connect(item, 'activate', Lang.bind(this, this._onLaunchActivate));
         this.addMenuItem(item);
 
+        if (Main.gpu_offload_supported) {
+            let item = new PopupMenu.PopupIconMenuItem(_("Run with NVIDIA GPU"), "cpu", St.IconType.SYMBOLIC);
+            this._signals.connect(item, 'activate', Lang.bind(this, this._onLaunchOffloadedActivate));
+            this.addMenuItem(item);
+        }
+
         item = new PopupMenu.PopupIconMenuItem(_("Add"), "list-add", St.IconType.SYMBOLIC);
         this._signals.connect(item, 'activate', Lang.bind(this, this._onAddActivate));
         this.addMenuItem(item);
@@ -76,6 +84,10 @@ class PanelAppLauncherMenu extends Applet.AppletPopupMenu {
         this._launcher.launch();
     }
 
+    _onLaunchOffloadedActivate(item, event) {
+        this._launcher.launch(true);
+    }
+
     _onRemoveActivate(item, event) {
         this.close();
         this._launcher.launchersBox.removeLauncher(this._launcher, this._launcher.isCustom());
@@ -219,12 +231,19 @@ class PanelAppLauncher extends DND.LauncherDraggable {
                          });
     }
 
-    launch() {
+    launch(offload=false) {
         if (this.isCustom()) {
             this.appinfo.launch([], null);
-        }
-        else {
-            this.app.open_new_window(-1);
+        } else {
+            if (offload) {
+                try {
+                    this.app.launch_offloaded(0, [], -1);
+                } catch (e) {
+                    logError(e, "Could not launch app with dedicated gpu: ");
+                }
+            } else {
+                this.app.open_new_window(-1);
+            }
         }
         this._animateIcon(0);
     }
@@ -327,7 +346,7 @@ class LaunchersBox {
 
         let vertical = this.applet.orientation == St.Side.LEFT || this.applet.orientation == St.Side.RIGHT;
         this._dragPlaceholder = new DND.GenericDragPlaceholderItem();
-        let placeholderSize = vertical ? [1, this.applet.icon_size] : [this.applet.icon_size, 1];
+        let placeholderSize = vertical ? [1, this.applet.icon_size * global.ui_scale] : [this.applet.icon_size * global.ui_scale, 1];
         this._dragPlaceholder.child.set_size(...placeholderSize);
         this.actor.insert_child_at_index(this._dragPlaceholder.actor, index);
 
@@ -372,7 +391,11 @@ class LaunchersBox {
         let boxSize = vertical ? this.actor.height : this.actor.width;
         let mPos = vertical ? y : x;
         let children = this.actor.get_children();
-        let dropIndex = Math.round(mPos / boxSize * children.length);
+
+        if(!vertical && St.Widget.get_default_direction () === St.TextDirection.RTL) //in RTL the dropIndex should be reversed
+            mPos = boxSize - mPos;
+
+        let dropIndex = Math.floor(mPos / boxSize * children.length);
 
         // -1 is end, 0 is start
         if (dropIndex >= children.length)
@@ -556,7 +579,7 @@ class CinnamonPanelLaunchersApplet extends Applet.Applet {
         let app = appSys.lookup_app(path);
         let appinfo = null;
         if (!app) {
-            appinfo = Gio.DesktopAppInfo.new_from_filename(CUSTOM_LAUNCHERS_PATH+"/"+path);
+            appinfo = CMenu.DesktopAppInfo.new_from_filename(CUSTOM_LAUNCHERS_PATH+"/"+path);
             if (!appinfo) {
                 global.logWarning(`Failed to add launcher from path: ${path}`);
                 return null;
