@@ -64,6 +64,7 @@ const Tooltips = imports.ui.tooltips;
 
 const MAX_TEXT_LENGTH = 1000;
 const FLASH_INTERVAL = 500;
+const FLASH_MAX_COUNT = 4;
 
 const WINDOW_PREVIEW_WIDTH = 200;
 const WINDOW_PREVIEW_HEIGHT = 150;
@@ -253,7 +254,7 @@ class WindowPreview extends Tooltips.TooltipBase {
 }
 
 class AppMenuButton {
-    constructor(applet, metaWindow, alert) {
+    constructor(applet, metaWindow, transient) {
         this.actor = new Cinnamon.GenericContainer({
             name: 'appMenu',
             style_class: 'window-list-item-box',
@@ -263,9 +264,11 @@ class AppMenuButton {
 
         this._applet = applet;
         this.metaWindow = metaWindow;
-        this.alert = alert;
+        this.transient = transient;
+        let initially_urgent = transient || metaWindow.is_demanding_attention() || metaWindow.is_urgent();
         this.labelVisible = false;
         this._signals = new SignalManager.SignalManager();
+        this.xid = metaWindow.get_xwindow();
 
         if (this._applet.orientation == St.Side.TOP)
             this.actor.add_style_class_name('top');
@@ -334,7 +337,7 @@ class AppMenuButton {
 
         this.onPreviewChanged();
 
-        if (!this.alert) {
+        if (!this.transient) {
             this._menuManager = new PopupMenu.PopupMenuManager(this);
             this.rightClickMenu = new AppMenuButtonRightClickMenu(this, this.metaWindow, this._applet.orientation);
             this._menuManager.addMenu(this.rightClickMenu);
@@ -358,7 +361,7 @@ class AppMenuButton {
         this.onFocus();
         this.setIcon();
 
-        if (this.alert)
+        if (initially_urgent)
             this.getAttention();
 
         this._signals.connect(this.metaWindow, 'notify::title', this.setDisplayTitle, this);
@@ -527,7 +530,7 @@ class AppMenuButton {
     destroy() {
         this._signals.disconnectAllSignals();
         this._tooltip.destroy();
-        if (!this.alert) {
+        if (!this.transient) {
             this.rightClickMenu.destroy();
             this._menuManager.destroy();
         }
@@ -559,7 +562,7 @@ class AppMenuButton {
             this.actor.remove_style_class_name("window-list-item-demands-attention-top");
             this._needsAttention = false;
 
-            if (this.alert) {
+            if (this.transient) {
                 this.destroy();
                 this._windows.splice(this._windows.indexOf(this), 1);
             }
@@ -570,7 +573,7 @@ class AppMenuButton {
 
     _onButtonRelease(actor, event) {
         this._tooltip.hide();
-        if (this.alert) {
+        if (this.transient) {
             if (event.get_button() == 1)
                 this._toggleWindow(false);
             return false;
@@ -589,7 +592,7 @@ class AppMenuButton {
 
     _onButtonPress(actor, event) {
         this._tooltip.hide();
-        if (!this.alert && event.get_button() == 3) {
+        if (!this.transient && event.get_button() == 3) {
             this.rightClickMenu.mouseEvent = event;
             this.rightClickMenu.toggle();
 
@@ -736,7 +739,7 @@ class AppMenuButton {
     }
 
     updateLabelVisible() {
-        if (this._applet.orientation == St.Side.TOP || this._applet.orientation == St.Side.BOTTOM) {
+        if (this._applet.showLabelPanel) {
             this._label.show();
             this.labelVisible = true;
         } else {
@@ -788,9 +791,8 @@ class AppMenuButton {
                 this.actor.remove_style_class_name(sc);
             else
                 this.actor.add_style_class_name(sc);
-            let result = counter < 4;
-            counter++;
-            return result;
+
+            return counter++ < FLASH_MAX_COUNT;
         });
     }
 };
@@ -945,6 +947,7 @@ class AppMenuButtonRightClickMenu extends Applet.AppletPopupMenu {
             });
         }
         this.addMenuItem(item);
+        item.setSensitive(mw.can_maximize())
 
         item = new PopupMenu.PopupIconMenuItem(_("Close"), "edit-delete", St.IconType.SYMBOLIC);
         this._signals.connect(item, 'activate', function() {
@@ -1015,9 +1018,11 @@ class CinnamonWindowListApplet extends Applet.Applet {
         this.settings.bind("left-click-minimize", "leftClickMinimize");
         this.settings.bind("middle-click-close", "middleClickClose");
         this.settings.bind("buttons-use-entire-space", "buttonsUseEntireSpace", this._refreshAllItems);
+        this.settings.bind("panel-show-label", "showLabelPanel", this._updateLabels);
         this.settings.bind("window-preview", "usePreview", this._onPreviewChanged);
         this.settings.bind("window-preview-show-label", "showLabel", this._onPreviewChanged);
         this.settings.bind("window-preview-scale", "previewScale", this._onPreviewChanged);
+        this.settings.bind("last-window-order", "lastWindowOrder", null);
 
         this.signals.connect(global.screen, 'window-added', this._onWindowAddedAsync, this);
         this.signals.connect(global.screen, 'window-monitor-changed', this._onWindowMonitorChanged, this);
@@ -1205,7 +1210,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
          * AppMenuButton. If this is actually a temporary AppMenuButton for
          * urgent windows on other workspaces, it is shown iff the normal
          * one isn't shown! */
-        if (window.alert)
+        if (window.transient)
             window.actor.visible = !window.actor.visible;
 
         if (window.actor.visible)
@@ -1223,6 +1228,11 @@ class CinnamonWindowListApplet extends Applet.Applet {
             window.setDisplayTitle();
         }
     }
+    
+    _updateLabels() {
+        for (let window of this._windows)
+            window.updateLabelVisible();
+    }
 
     _updateWatchedMonitors() {
         let n_mons = Gdk.Screen.get_default().get_n_monitors();
@@ -1263,6 +1273,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
             }
         }
 
+        this.refreshing = true;
 
         for (let window of windows) {
             if (this._shouldAdd(window))
@@ -1270,15 +1281,19 @@ class CinnamonWindowListApplet extends Applet.Applet {
             else
                 this._removeWindow(window);
         }
+
+        this.refreshing = false;
+
+        this._applySavedOrder();
     }
 
-    _addWindow(metaWindow, alert) {
+    _addWindow(metaWindow, transient) {
         for (let window of this._windows)
             if (window.metaWindow == metaWindow &&
-                window.alert == alert)
+                window.transient == transient)
                 return;
 
-        let appButton = new AppMenuButton(this, metaWindow, alert);
+        let appButton = new AppMenuButton(this, metaWindow, transient);
         this.manager_container.add_actor(appButton.actor);
 
         this._windows.push(appButton);
@@ -1287,7 +1302,7 @@ class CinnamonWindowListApplet extends Applet.Applet {
          * workspace. So if we add an AppMenuButton for a window in another
          * workspace, put it in the right position. It is at the end by
          * default, so move it to the start if needed */
-        if (alert) {
+        if (transient) {
             if (metaWindow.get_workspace().index() < global.screen.get_active_workspace_index())
                 this.manager_container.set_child_at_index(appButton.actor, 0);
         } else {
@@ -1297,6 +1312,8 @@ class CinnamonWindowListApplet extends Applet.Applet {
                 }
             }
         }
+
+        this._saveOrder();
     }
 
     _removeWindow(metaWindow) {
@@ -1308,6 +1325,8 @@ class CinnamonWindowListApplet extends Applet.Applet {
                 this._windows.splice(i, 1);
             }
         }
+
+        this._saveOrder();
     }
 
     _shouldAdd(metaWindow) {
@@ -1316,6 +1335,52 @@ class CinnamonWindowListApplet extends Applet.Applet {
             this._monitorWatchList.indexOf(metaWindow.get_monitor()) != -1;
     }
 
+    /* Store by Windows (XIDs), a simple list
+       xid::xid::xid::xid::xid
+    */
+
+    _applySavedOrder() {
+        let order = this.lastWindowOrder.split("::");
+
+        order.reverse();
+
+        for (let i = 0; i < order.length; i++) {
+            let xid = parseInt(order[i]);
+
+            if (xid === NaN) {
+                continue;
+            }
+
+            let found = this._windows.find(win => (win.xid == xid));
+
+            if (found) {
+                this.manager_container.set_child_at_index(found.actor, 0);
+            }
+        }
+
+        this._saveOrder();
+    }
+
+    _saveOrder() {
+        if (this.refreshing) {
+            return;
+        }
+
+        let new_order = [];
+        let actors = this.manager_container.get_children();
+
+        for (let i = 0; i < actors.length; i++) {
+            new_order.push(actors[i]._delegate.xid);
+        }
+
+        if (new_order.length === 0) {
+            this.lastWindowOrder = "";
+            return;
+        }
+
+        this.lastWindowOrder = new_order.join("::");
+    }
+
     handleDragOver(source, actor, x, y, time) {
         if (this._inEditMode)
             return DND.DragMotionResult.MOVE_DROP;
@@ -1323,21 +1388,19 @@ class CinnamonWindowListApplet extends Applet.Applet {
             return DND.DragMotionResult.NO_DROP;
 
         let children = this.manager_container.get_children();
-        let isVertical = this.manager_container.height > this.manager_container.width;
+        let isVertical = (this.orientation == St.Side.LEFT || this.orientation == St.Side.RIGHT);
+        let axis = isVertical ? [y, 'y1'] : [x, 'x1'];
 
-        this._dragPlaceholderPos = -1
-        for (let i = children.length - 1; i >= 0; i--) {
+        this._dragPlaceholderPos = -1;
+        let minDist = -1;
+        for(let i = 0; i < children.length; i++) {
             if (!children[i].visible)
                 continue;
-
-            if (isVertical) {
-                if (y > children[i].get_allocation_box().y1) {
-                    this._dragPlaceholderPos = i;
-                    break;
-                }
-            } else if  (x > children[i].get_allocation_box().x1) {
+            let dim = isVertical ? children[i].height : children[i].width;
+            let dist = Math.abs(axis[0] - (children[i].get_allocation_box()[axis[1]] + dim / 2));
+            if(dist < minDist || minDist == -1) {
+                minDist = dist;
                 this._dragPlaceholderPos = i;
-                break;
             }
         }
 
@@ -1363,6 +1426,8 @@ class CinnamonWindowListApplet extends Applet.Applet {
 
         this.manager_container.set_child_at_index(source.actor, this._dragPlaceholderPos);
 
+        this._saveOrder();
+
         return true;
     }
 
