@@ -270,7 +270,8 @@ class CinnamonWorkspaceSwitcher extends Applet.Applet {
             this._focusWindow = global.display.focus_window.get_compositor_private();
 
         this.settings = new Settings.AppletSettings(this, metadata.uuid, instance_id);
-        this.settings.bind("display_type", "display_type", this.queueCreateButtons);
+        this.settings.bind("display-type", "display_type", this.queueCreateButtons);
+        this.settings.bind("scroll-behavior", "scroll_behavior");
 
         this.actor.connect('scroll-event', this.hook.bind(this));
 
@@ -353,6 +354,9 @@ class CinnamonWorkspaceSwitcher extends Applet.Applet {
     }
 
     hook(actor, event) {
+        if (this.scroll_behavior == "disabled")
+            return;
+
         let now = (new Date()).getTime();
         let direction = event.get_scroll_direction();
 
@@ -364,8 +368,11 @@ class CinnamonWorkspaceSwitcher extends Applet.Applet {
         if ((now - this._last_switch) > MIN_SWITCH_INTERVAL_MS ||
             direction !== this._last_switch_direction) {
 
-            if (direction == 0) Main.wm.actionMoveWorkspaceLeft();
-            else if (direction == 1) Main.wm.actionMoveWorkspaceRight();
+            // XOR used to determine the effective direction
+            if ((direction == 0) == (this.scroll_behavior == "normal"))
+                Main.wm.actionMoveWorkspaceLeft();
+            else
+                Main.wm.actionMoveWorkspaceRight();
 
             this._last_switch = now;
             this._last_switch_direction = direction;
