@@ -1,12 +1,11 @@
 #!/usr/bin/python3
 
-from gi.repository import Gio, Gtk, GObject, Gdk
 import html
 import gettext
 
 import gi
 gi.require_version("Gtk", "3.0")
-from gi.repository import Gio, Gtk, GObject, Gdk
+from gi.repository import Gio, Gtk
 
 from KeybindingWidgets import CellRendererKeybinding
 from SettingsWidgets import SidePage
@@ -177,6 +176,7 @@ KEYBINDINGS = [
     [_("Volume mute"), MEDIA_KEYS_SCHEMA, "volume-mute", "media"],
     [_("Volume down"), MEDIA_KEYS_SCHEMA, "volume-down", "media"],
     [_("Volume up"), MEDIA_KEYS_SCHEMA, "volume-up", "media"],
+    [_("Mic mute"), MEDIA_KEYS_SCHEMA, "mic-mute", "media"],
     [_("Launch media player"), MEDIA_KEYS_SCHEMA, "media", "media"],
     [_("Play"), MEDIA_KEYS_SCHEMA, "play", "media"],
     [_("Pause playback"), MEDIA_KEYS_SCHEMA, "pause", "media"],
@@ -202,6 +202,26 @@ KEYBINDINGS = [
     [_("High contrast on or off"), MEDIA_KEYS_SCHEMA, "toggle-contrast", "accessibility"]
 ]
 
+# keybindings.js listens for changes to 'custom-list'. Any time we create a shortcut
+# or add/remove individual keybindings, we need to cause this list to change.
+#
+# Unfortunately, at some point recently, simply 'touching' the setting without actually
+# modifying its contents stopped working (see CustomKeybinding.writeSettings), and
+# we must now do something more substantial, like reversing the list each time (the order
+# doesn't matter to the rest of this code).  In order for this to be reliable we need
+# to make sure we always end up with at least 2 entries in 'custom-list', since reversing
+# a list with one element won't do anything.
+DUMMY_CUSTOM_ENTRY = "__dummy__"
+
+def ensureCustomListIsValid(custom_list):
+    if len(custom_list) > 1:
+        return;
+
+    if DUMMY_CUSTOM_ENTRY in custom_list:
+        return;
+
+    custom_list.append(DUMMY_CUSTOM_ENTRY);
+
 class Module:
     comment = _("Manage keyboard settings and shortcuts")
     name = "keyboard"
@@ -419,7 +439,7 @@ class Module:
             categories, iter = tree.get_selection().get_selected()
             if iter:
                 category = categories[iter][2]
-                if category.int_name is not "custom":
+                if category.int_name != "custom":
                     for keybinding in category.keybindings:
                         self.kb_store.append((keybinding.label, keybinding))
                 else:
@@ -428,13 +448,16 @@ class Module:
 
     def loadCustoms(self):
         for category in self.main_store:
-            if category.int_name is "custom":
+            if category.int_name == "custom":
                 category.clear()
 
         parent = Gio.Settings.new(CUSTOM_KEYS_PARENT_SCHEMA)
         custom_list = parent.get_strv("custom-list")
 
         for entry in custom_list:
+            if entry == DUMMY_CUSTOM_ENTRY:
+                continue
+
             custom_path = CUSTOM_KEYS_BASENAME+"/"+entry+"/"
             schema = Gio.Settings.new_with_path(CUSTOM_KEYS_SCHEMA, custom_path)
             custom_kb = CustomKeyBinding(entry,
@@ -443,7 +466,7 @@ class Module:
                                          schema.get_strv("binding"))
             self.kb_store.append((custom_kb.label, custom_kb))
             for category in self.main_store:
-                if category.int_name is "custom":
+                if category.int_name == "custom":
                     category.add(custom_kb)
 
     def onKeyBindingChanged(self, tree):
@@ -453,7 +476,7 @@ class Module:
             if iter:
                 keybinding = keybindings[iter][1]
                 for entry in keybinding.entries:
-                    if entry is not "_invalid_":
+                    if entry != "_invalid_":
                         self.entry_store.append((entry,))
                 self.remove_custom_button.set_property('sensitive', isinstance(keybinding, CustomKeyBinding))
 
@@ -515,6 +538,9 @@ class Module:
         array = parent.get_strv("custom-list")
         num_array = []
         for entry in array:
+            if entry == DUMMY_CUSTOM_ENTRY:
+                continue
+
             num_array.append(int(entry.replace("custom", "")))
         num_array.sort()
 
@@ -527,6 +553,7 @@ class Module:
 
         new_str = "custom" + str(i)
         array.append(new_str)
+        ensureCustomListIsValid(array);
         parent.set_strv("custom-list", array)
 
         new_path = CUSTOM_KEYS_BASENAME + "/custom" + str(i) + "/"
@@ -536,7 +563,7 @@ class Module:
         new_schema.set_strv("binding", ())
         i = 0
         for cat in self.cat_store:
-            if cat[2].int_name is "custom":
+            if cat[2].int_name == "custom":
                 self.cat_tree.set_cursor(str(i), self.cat_tree.get_column(0), False)
             i += 1
         i = 0
@@ -570,11 +597,12 @@ class Module:
                     break
             if existing:
                 array.remove(keybinding.path)
+                ensureCustomListIsValid(array)
                 parent_settings.set_strv("custom-list", array)
 
         i = 0
         for cat in self.cat_store:
-            if cat[2].int_name is "custom":
+            if cat[2].int_name == "custom":
                 self.cat_tree.set_cursor(str(i), self.cat_tree.get_column(0), False)
             i += 1
 
@@ -600,7 +628,7 @@ class Module:
 
                 i = 0
                 for cat in self.cat_store:
-                    if cat[2].int_name is "custom":
+                    if cat[2].int_name == "custom":
                         self.cat_tree.set_cursor(str(i), self.cat_tree.get_column(0), False)
                     i += 1
                 i = 0
@@ -644,7 +672,6 @@ class Module:
         keybinding.resetDefaults()
         self.onKeyBindingChanged(self.kb_tree)
 
-
 class KeyBindingCategory():
     def __init__(self, label, int_name, parent, icon):
         self.label = label
@@ -691,7 +718,7 @@ class KeyBinding():
     def writeSettings(self):
         array = []
         for entry in self.entries:
-            if entry is not "":
+            if entry != "":
                 array.append(entry)
         self.settings.set_strv(self.key, array)
 
@@ -731,13 +758,15 @@ class CustomKeyBinding():
 
         array = []
         for entry in self.entries:
-            if entry is not "":
+            if entry != "":
                 array.append(entry)
         settings.set_strv("binding", array)
 
         # Touch the custom-list key, this will trigger a rebuild in cinnamon
         parent = Gio.Settings.new(CUSTOM_KEYS_PARENT_SCHEMA)
         custom_list = parent.get_strv("custom-list")
+        custom_list.reverse()
+        ensureCustomListIsValid(custom_list);
         parent.set_strv("custom-list", custom_list)
 
 class AddCustomDialog(Gtk.Dialog):
@@ -777,6 +806,5 @@ class AddCustomDialog(Gtk.Dialog):
         self.command_entry.set_text(path)
 
     def onEntriesChanged(self, widget):
-        ok_enabled = self.name_entry.get_text().strip() is not "" and self.command_entry.get_text().strip() is not ""
+        ok_enabled = self.name_entry.get_text().strip() != "" and self.command_entry.get_text().strip() != ""
         self.set_response_sensitive(Gtk.ResponseType.OK, ok_enabled)
-        
\ No newline at end of file
