@@ -1,5 +1,9 @@
 #!/usr/bin/python3
 import getopt
+
+from bin import util
+util.strip_syspath_locals()
+
 import gi
 gi.require_version('Gtk', '3.0')
 gi.require_version('XApp', '1.0')
@@ -41,8 +45,6 @@ XLET_SETTINGS_WIDGETS = {
     "fontchooser"       :   "JSONSettingsFontButton",
     "soundfilechooser"  :   "JSONSettingsSoundFileChooser",
     "iconfilechooser"   :   "JSONSettingsIconChooser",
-    "tween"             :   "JSONSettingsTweenChooser",
-    "effect"            :   "JSONSettingsEffectChooser",
     "datechooser"       :   "JSONSettingsDateChooser",
     "timechooser"       :   "JSONSettingsTimeChooser",
     "keybinding"        :   "JSONSettingsKeybinding",
@@ -254,14 +256,15 @@ class MainWindow(object):
 
         if "icon" in self.xlet_meta:
             self.window.set_icon_name(self.xlet_meta["icon"])
+
+        icon_path = os.path.join(self.xlet_dir, "icon.svg")
+        if os.path.exists(icon_path):
+            self.window.set_icon_from_file(icon_path)
         else:
-            icon_path = os.path.join(self.xlet_dir, "icon.svg")
+            icon_path = os.path.join(self.xlet_dir, "icon.png")
             if os.path.exists(icon_path):
                 self.window.set_icon_from_file(icon_path)
-            else:
-                icon_path = os.path.join(self.xlet_dir, "icon.png")
-                if os.path.exists(icon_path):
-                    self.window.set_icon_from_file(icon_path)
+
         self.window.set_title(translate(self.uuid, self.xlet_meta["name"]))
 
         def check_sizing(widget, data=None):
