@@ -77,6 +77,9 @@ const CinnamonIface =
                 <arg type="s" direction="in" /> \
             </method> \
             <method name="induceSegfault" /> \
+            <method name="leakMemory"> \
+                <arg type="i" direction="in" name="mb" /> \
+            </method> \
             <method name="switchWorkspaceRight" /> \
             <method name="switchWorkspaceLeft" /> \
             <method name="switchWorkspaceUp" /> \
@@ -117,7 +120,12 @@ const CinnamonIface =
             <method name="GetRunState"> \
                <arg type="i" direction="out" name="state" /> \
             </method> \
+            <method name="RestartCinnamon"> \
+                <arg type="b" direction="in" name="show_osd" /> \
+            </method> \
+            <method name="ReloadTheme"/> \
             <signal name="RunStateChanged"/> \
+            <signal name="XletsLoadedComplete"/> \
         </interface> \
     </node>';
 
@@ -191,7 +199,7 @@ CinnamonDBus.prototype = {
      */
     ScreenshotArea: function(include_cursor, x, y, width, height, flash, filename) {
         let screenshot = new Cinnamon.Screenshot();
-        screenshot.screenshot_area(include_cursor, x, y, width, 200, filename,
+        screenshot.screenshot_area(include_cursor, x, y, width, height, filename,
             Lang.bind(this, this._onScreenshotComplete, flash));
     },
 
@@ -362,6 +370,10 @@ CinnamonDBus.prototype = {
         global.segfault();
     },
 
+    leakMemory: function(mb) {
+        global.alloc_leak(mb);
+    },
+
     switchWorkspaceLeft: function() {
         Main.wm.actionMoveWorkspaceLeft();
     },
@@ -453,6 +465,14 @@ CinnamonDBus.prototype = {
         return Main.runState;
     },
 
+    RestartCinnamon: function(showOsd) {
+        Main.restartCinnamon(showOsd);
+    },
+
+    ReloadTheme: function() {
+        Main.themeManager._changeTheme()
+    },
+
     EmitRunStateChanged: function() {
         this._dbusImpl.emit_signal('RunStateChanged', null);
     },
@@ -461,5 +481,9 @@ CinnamonDBus.prototype = {
         this._dbusImpl.emit_signal('MonitorsChanged', null);
     },
 
+    EmitXletsLoadedComplete: function() {
+        this._dbusImpl.emit_signal('XletsLoadedComplete', null);
+    },
+
     CinnamonVersion: Config.PACKAGE_VERSION
 };
