@@ -10,9 +10,10 @@
  * @wm (WindowManager.WindowManager): The window manager
  * @messageTray (MessageTray.MessageTray): The mesesage tray
  * @notificationDaemon (NotificationDaemon.NotificationDaemon): The notification daemon
- * @windowAttentionHandler (WindowAttentionHandler.WindowAttentionHandler): The window attention handler
+ * @windowAttentionHandler (WindowAttentionHandler.WindowAttentionHandler): The window attention handle
  * @recorder (Cinnamon.Recorder): The recorder
  * @cinnamonDBusService (CinnamonDBus.Cinnamon): The cinnamon dbus object
+ * @screenshotService (Screenshot.ScreenshotService): Implementation of gnome-shell's screenshot interface.
  * @modalCount (int): The number of modals "pushed"
  * @modalActorFocusStack (array): Array of pushed modal actors
  * @uiGroup (Cinnamon.GenericContainer): The group containing all Cinnamon and
@@ -103,6 +104,7 @@ const LookingGlass = imports.ui.lookingGlass;
 const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
 const CinnamonDBus = imports.ui.cinnamonDBus;
+const Screenshot = imports.ui.screenshot;
 const ThemeManager = imports.ui.themeManager;
 const Magnifier = imports.ui.magnifier;
 const XdndHandler = imports.ui.xdndHandler;
@@ -140,6 +142,7 @@ var notificationDaemon = null;
 var windowAttentionHandler = null;
 var recorder = null;
 var cinnamonDBusService = null;
+var screenshotService = null;
 var modalCount = 0;
 var modalActorFocusStack = [];
 var uiGroup = null;
@@ -310,6 +313,8 @@ function start() {
     cinnamonDBusService = new CinnamonDBus.CinnamonDBus();
     setRunState(RunState.STARTUP);
 
+    screenshotService = new Screenshot.ScreenshotService();
+
     // Ensure CinnamonWindowTracker and CinnamonAppUsage are initialized; this will
     // also initialize CinnamonAppSystem first.  CinnamonAppSystem
     // needs to load all the .desktop files, and CinnamonWindowTracker
@@ -422,7 +427,6 @@ function start() {
     keyboard = new Keyboard.Keyboard();
     notificationDaemon = new NotificationDaemon.NotificationDaemon();
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
-
     placesManager = new PlacesManager.PlacesManager();
 
     magnifier = new Magnifier.Magnifier();
@@ -1038,7 +1042,7 @@ function _logInfo(msg) {
         // Convert arguments to an array, add 'info' to the beginning of it. Invoke _log with apply so
         // unlimited arguments can be passed to it.
         let args = Array.prototype.slice.call(arguments);
-        _log.apply(this, ['info'].concat(args));
+        _log.apply(this, ['info', ...args]);
     }
 }
 
@@ -1321,30 +1325,32 @@ function getRunDialog() {
  * activateWindow:
  * @window (Meta.Window): the Meta.Window to activate
  * @time (int): (optional) current event time
- * @workspaceNum (int): (optional) window's workspace number
+ * @workspaceNum (int): (optional) workspace number to switch to
  *
- * Activates @window, switching to its workspace first if necessary,
- * and switching out of the overview if it's currently active
+ * Activates @window, switching to workspaceNum first if provided,
+ * and switching out of the overview if it's currently active. If
+ * no workspace is provided, workspace-related behavior during
+ * activation will be handled in muffin.
  */
 function activateWindow(window, time, workspaceNum) {
     let activeWorkspaceNum = global.screen.get_active_workspace_index();
-    let windowWorkspaceNum = (workspaceNum !== undefined) ? workspaceNum : window.get_workspace().index();
 
     if (!time)
         time = global.get_current_time();
 
-    if (windowWorkspaceNum != activeWorkspaceNum) {
-        let workspace = global.screen.get_workspace_by_index(windowWorkspaceNum);
+    if ((workspaceNum !== undefined) && activeWorkspaceNum !== workspaceNum) {
+        let workspace = global.screen.get_workspace_by_index(workspaceNum);
         workspace.activate_with_focus(window, time);
-    } else {
-        window.activate(time);
-        Mainloop.idle_add(function() {
-            window.foreach_transient(function(win) {
-                win.activate(time);
-            });
-        });
+        return;
     }
 
+    window.activate(time);
+    Mainloop.idle_add(function() {
+        window.foreach_transient(function(win) {
+            win.activate(time);
+        });
+    });
+
     overview.hide();
     expo.hide();
 }
