@@ -3,6 +3,7 @@
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const Pango = imports.gi.Pango;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -164,9 +165,11 @@ __proto__: ModalDialog.ModalDialog.prototype,
         global.display.connect('restart', () => this.close());
 
         let label = new St.Label({ style_class: 'run-dialog-label',
-                                   text: _("Please enter a command:") });
+                                   text: _("Enter a command") });
 
-        this.contentLayout.add(label, { y_align: St.Align.START });
+        this.contentLayout.set_width(350 * global.ui_scale);
+
+        this.contentLayout.add(label, { x_align: St.Align.MIDDLE });
 
         let entry = new St.Entry({ style_class: 'run-dialog-entry' });
         CinnamonEntry.addContextMenu(entry);
@@ -182,25 +185,16 @@ __proto__: ModalDialog.ModalDialog.prototype,
         this.contentLayout.add(this._completionBox);
         this._completionSelected = 0;
 
-        this._errorBox = new St.BoxLayout({ style_class: 'run-dialog-error-box' });
+        let defaultDescriptionText = _("Press ESC to close");
 
-        this.contentLayout.add(this._errorBox, { expand: true });
-
-        let errorIcon = new St.Icon({ icon_name: 'dialog-error', icon_size: 24, style_class: 'run-dialog-error-icon' });
-
-        this._errorBox.add(errorIcon, { y_align: St.Align.MIDDLE });
+        this._descriptionLabel = new St.Label({ style_class: 'run-dialog-description',
+                                                text:        defaultDescriptionText });
+        this._descriptionLabel.clutter_text.line_wrap = true;
+        this._descriptionLabel.clutter_text.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR);
+        this.contentLayout.add(this._descriptionLabel, { y_align: St.Align.MIDDLE });
 
         this._commandError = false;
 
-        this._errorMessage = new St.Label({ style_class: 'run-dialog-error-label' });
-        this._errorMessage.clutter_text.line_wrap = true;
-
-        this._errorBox.add(this._errorMessage, { expand: true,
-                                                 y_align: St.Align.MIDDLE,
-                                                 y_fill: false });
-
-        this._errorBox.hide();
-
         this._entryText.connect('key-press-event', Lang.bind(this, this._onKeyPress));
 
         this._history = new History.HistoryManager({ gsettingsKey: HISTORY_KEY,
@@ -212,7 +206,7 @@ __proto__: ModalDialog.ModalDialog.prototype,
 
     _onKeyPress: function (o, e) {
         let symbol = e.get_key_symbol();
-        if (symbol == Clutter.Return || symbol == Clutter.KP_Enter) {
+        if (symbol === Clutter.KEY_Return || symbol === Clutter.KEY_KP_Enter) {
             if (o.get_text().trim() == "") {
                 return false;
             }
@@ -243,26 +237,26 @@ __proto__: ModalDialog.ModalDialog.prototype,
             }
             return true;
         }
-        if (symbol == Clutter.Escape || symbol == Clutter.Super_L || symbol == Clutter.Super_R) {
+        if (symbol === Clutter.KEY_Escape || symbol === Clutter.KEY_Super_L || symbol === Clutter.KEY_Super_R) {
             this.close();
             return true;
         }
-        if (symbol == Clutter.Tab) {
+        if (symbol === Clutter.KEY_Tab) {
             this._updateCompletions(NAVIGATE_TYPE_TAB);
             return true;
         }
 
         if (this._completionBox.visible) {
-            if (symbol == Clutter.KEY_Up) {
+            if (symbol === Clutter.KEY_Up) {
                 this._updateCompletions(NAVIGATE_TYPE_ARROW, UP);
                 return true;
-            } else if (symbol == Clutter.KEY_Down) {
+            } else if (symbol === Clutter.KEY_Down) {
                 this._updateCompletions(NAVIGATE_TYPE_ARROW, DOWN);
                 return true;
             }
         }
 
-        if (symbol == Clutter.BackSpace) {
+        if (symbol === Clutter.KEY_BackSpace) {
             this._completionSelected = 0;
             this._completionBox.hide();
             this._oldText = "";
@@ -463,28 +457,14 @@ __proto__: ModalDialog.ModalDialog.prototype,
     _showError : function(message) {
         this._commandError = true;
 
-        this._errorMessage.set_text(message.trim());
-
-        if (!this._errorBox.visible) {
-            let [errorBoxMinHeight, errorBoxNaturalHeight] = this._errorBox.get_preferred_height(-1);
-
-            let parentActor = this._errorBox.get_parent();
-            Tweener.addTween(parentActor,
-                             { height: parentActor.height + errorBoxNaturalHeight,
-                               time: DIALOG_GROW_TIME,
-                               transition: 'easeOutQuad',
-                               onComplete: Lang.bind(this,
-                                                     function() {
-                                                         parentActor.set_height(-1);
-                                                         this._errorBox.show();
-                                                     })
-                             });
-        }
+        this._descriptionLabel.set_text(message.trim());
+        this._descriptionLabel.add_style_class_name('error');
     },
 
     open: function() {
         this._history.lastItem();
-        this._errorBox.hide();
+        this._descriptionLabel.set_text(_("Press ESC to close"));
+        this._descriptionLabel.remove_style_class_name('error');
         this._entryText.set_text('');
         this._completionBox.hide();
         this._commandError = false;
