@@ -2,96 +2,11 @@
 
 #include "config.h"
 
-#include <dirent.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <math.h>
-#include <stdarg.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#ifdef HAVE_SYS_RESOURCE_H
-#include <sys/resource.h>
-#endif
-
-#include <X11/extensions/Xfixes.h>
-#include <cogl-pango/cogl-pango.h>
-#include <canberra.h>
-#include <clutter/glx/clutter-glx.h>
-#include <clutter/x11/clutter-x11.h>
-#include <gdk/gdkx.h>
-#include <gio/gio.h>
-#include <cjs/gjs-module.h>
-#include <girepository.h>
-#include <meta/display.h>
-#include <meta/util.h>
-
-/* Memory report bits */
-#ifdef HAVE_MALLINFO
-#include <malloc.h>
-#endif
-
-#include "cinnamon-enum-types.h"
 #include "cinnamon-global-private.h"
-#include "cinnamon-jsapi-compat-private.h"
-#include "cinnamon-marshal.h"
-#include "cinnamon-perf-log.h"
-#include "cinnamon-window-tracker.h"
-#include "cinnamon-wm.h"
-#include "st.h"
 
 static CinnamonGlobal *the_object = NULL;
 
 static void grab_notify (GtkWidget *widget, gboolean is_grab, gpointer user_data);
-static void cinnamon_global_on_gc (GjsContext   *context,
-                                CinnamonGlobal  *global);
-
-struct _CinnamonGlobal {
-  GObject parent;
-
-  ClutterStage *stage;
-  Window stage_xwindow;
-  GdkWindow *stage_gdk_window;
-
-  MetaDisplay *meta_display;
-  GdkDisplay *gdk_display;
-  Display *xdisplay;
-  MetaScreen *meta_screen;
-  GdkScreen *gdk_screen;
-
-  /* We use this window to get a notification from GTK+ when
-   * a widget in our process does a GTK+ grab.  See
-   * http://bugzilla.gnome.org/show_bug.cgi?id=570641
-   * 
-   * This window is never mapped or shown.
-   */
-  GtkWindow *grab_notifier;
-  gboolean gtk_grab_active;
-
-  CinnamonStageInputMode input_mode;
-  XserverRegion input_region;
-
-  GjsContext *js_context;
-  MetaPlugin *plugin;
-  CinnamonWM *wm;
-  GSettings *settings;
-  GSettings *interface_settings;
-  const char *datadir;
-  const char *imagedir;
-  const char *userdatadir;
-  StFocusManager *focus_manager;
-
-  guint work_count;
-  GSList *leisure_closures;
-  guint leisure_function_id;
-
-  /* For sound notifications */
-  ca_context *sound_context;
-
-  guint32 xdnd_timestamp;
-  gint64 last_gc_end_time;
-  guint ui_scale;
-};
 
 enum {
   PROP_0,
@@ -114,7 +29,8 @@ enum {
   PROP_IMAGEDIR,
   PROP_USERDATADIR,
   PROP_FOCUS_MANAGER,
-  PROP_UI_SCALE
+  PROP_UI_SCALE,
+  PROP_SESSION_RUNNING
 };
 
 /* Signals */
@@ -125,6 +41,7 @@ enum
  XDND_ENTER,
  NOTIFY_ERROR,
  SCALE_CHANGED,
+ SHUTDOWN,
  LAST_SIGNAL
 };
 
@@ -145,6 +62,9 @@ cinnamon_global_set_property(GObject         *object,
     case PROP_STAGE_INPUT_MODE:
       cinnamon_global_set_stage_input_mode (global, g_value_get_enum (value));
       break;
+    case PROP_SESSION_RUNNING:
+      global->session_running = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -228,12 +148,51 @@ cinnamon_global_get_property(GObject         *object,
     case PROP_UI_SCALE:
       g_value_set_uint (value, global->ui_scale);
       break;
+    case PROP_SESSION_RUNNING:
+      g_value_set_boolean (value, global->session_running);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
     }
 }
 
+static void
+failed_to_own_notifications (GDBusConnection *connection,
+                             const gchar     *name,
+                             gpointer         user_data)
+{
+    CinnamonGlobal *global = CINNAMON_GLOBAL (user_data);
+
+    g_message ("Tried to become the session notification handler but failed. "
+               "Maybe some other process is handling it.");
+}
+
+static void
+setup_notifications_service (CinnamonGlobal *global)
+{
+  guint owner_id;
+  gboolean disabled;
+
+  /* notify-osd allows itself to be replaced as the notification handler. dunst does not,
+     nor does cinnamon. If we attempt to own and fail, cinnamon is still 'on deck' to take
+     over if the existing handler disappears - our owner_id is still valid. */
+
+  disabled = g_settings_get_boolean (global->settings, "allow-other-notification-handlers");
+
+  if (disabled)
+  {
+    return;
+  }
+
+  owner_id = g_bus_own_name (G_BUS_TYPE_SESSION,
+                             "org.freedesktop.Notifications",
+                             G_BUS_NAME_OWNER_FLAGS_REPLACE,
+                             NULL, NULL,
+                             (GBusNameLostCallback) failed_to_own_notifications,
+                             global, NULL);
+}
+
 static void
 cinnamon_global_init (CinnamonGlobal *global)
 {
@@ -264,6 +223,8 @@ cinnamon_global_init (CinnamonGlobal *global)
 
   global->settings = g_settings_new ("org.cinnamon");
 
+  setup_notifications_service (global);
+
   global->ui_scale = 1;
 
   global->grab_notifier = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
@@ -272,19 +233,13 @@ cinnamon_global_init (CinnamonGlobal *global)
 
   global->input_mode = CINNAMON_STAGE_INPUT_MODE_NORMAL;
 
-  ca_context_create (&global->sound_context);
-  ca_context_change_props (global->sound_context, CA_PROP_APPLICATION_NAME, PACKAGE_NAME, CA_PROP_APPLICATION_ID, "org.Cinnamon", NULL);
-  ca_context_open (global->sound_context);
-
   if (!cinnamon_js)
     cinnamon_js = JSDIR;
   search_path = g_strsplit (cinnamon_js, ":", -1);
   global->js_context = g_object_new (GJS_TYPE_CONTEXT,
+                                     "profiler-sigusr2", true,
                                      "search-path", search_path,
-                                     "js-version", "1.8",
-                                     "gc-notifications", TRUE,
                                      NULL);
-  g_signal_connect (global->js_context, "gc", G_CALLBACK (cinnamon_global_on_gc), global);
 
   g_strfreev (search_path);
 }
@@ -293,11 +248,10 @@ static void
 cinnamon_global_finalize (GObject *object)
 {
   CinnamonGlobal *global = CINNAMON_GLOBAL (object);
-
   g_object_unref (global->js_context);
+
   gtk_widget_destroy (GTK_WIDGET (global->grab_notifier));
   g_object_unref (global->settings);
-  g_object_unref (global->interface_settings);
 
   the_object = NULL;
 
@@ -319,8 +273,7 @@ cinnamon_global_class_init (CinnamonGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    _cinnamon_marshal_VOID__INT_INT,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 
   /* Emitted from cinnamon-plugin.c during event handling */
@@ -329,8 +282,7 @@ cinnamon_global_class_init (CinnamonGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 0);
 
   /* Emitted from cinnamon-plugin.c during event handling */
@@ -339,8 +291,7 @@ cinnamon_global_class_init (CinnamonGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 0);
 
   cinnamon_global_signals[NOTIFY_ERROR] =
@@ -348,8 +299,7 @@ cinnamon_global_class_init (CinnamonGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    gi_cclosure_marshal_generic,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 2,
                     G_TYPE_STRING,
                     G_TYPE_STRING);
@@ -359,8 +309,15 @@ cinnamon_global_class_init (CinnamonGlobalClass *klass)
                     G_TYPE_FROM_CLASS (klass),
                     G_SIGNAL_RUN_LAST,
                     0,
-                    NULL, NULL,
-                    g_cclosure_marshal_VOID__VOID,
+                    NULL, NULL, NULL,
+                    G_TYPE_NONE, 0);
+
+  cinnamon_global_signals[SHUTDOWN] =
+      g_signal_new ("shutdown",
+                    G_TYPE_FROM_CLASS (klass),
+                    G_SIGNAL_RUN_LAST,
+                    0,
+                    NULL, NULL, NULL,
                     G_TYPE_NONE, 0);
 
   g_object_class_install_property (gobject_class,
@@ -463,28 +420,28 @@ cinnamon_global_class_init (CinnamonGlobalClass *klass)
                                    PROP_SETTINGS,
                                    g_param_spec_object ("settings",
                                                         "Settings",
-                                                        "GSettings instance for cinnamon configuration",
+                                                        "GSettings instance for Cinnamon configuration",
                                                         G_TYPE_SETTINGS,
                                                         G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_DATADIR,
                                    g_param_spec_string ("datadir",
                                                         "Data directory",
-                                                        "Directory containing cinnamon data files",
+                                                        "Directory containing Cinnamon data files",
                                                         NULL,
                                                         G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_IMAGEDIR,
                                    g_param_spec_string ("imagedir",
                                                         "Image directory",
-                                                        "Directory containing cinnamon image files",
+                                                        "Directory containing Cinnamon image files",
                                                         NULL,
                                                         G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
                                    PROP_USERDATADIR,
                                    g_param_spec_string ("userdatadir",
                                                         "User data directory",
-                                                        "Directory containing cinnamon user data",
+                                                        "Directory containing Cinnamon user data",
                                                         NULL,
                                                         G_PARAM_READABLE));
   g_object_class_install_property (gobject_class,
@@ -502,6 +459,14 @@ cinnamon_global_class_init (CinnamonGlobalClass *klass)
                                                       "Current UI Scale",
                                                       0, G_MAXUINT, 1,
                                                       G_PARAM_READABLE));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_SESSION_RUNNING,
+                                   g_param_spec_boolean ("session-running",
+                                                         "Session state",
+                                                         "If the session startup has already finished",
+                                                         FALSE,
+                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE));
 }
 
 /**
@@ -646,11 +611,41 @@ cinnamon_global_set_cursor (CinnamonGlobal *global,
       name = "dnd-copy";
       break;
     case CINNAMON_CURSOR_DND_UNSUPPORTED_TARGET:
-      name = "dnd-none";
+      name = "X_cursor";
       break;
     case CINNAMON_CURSOR_POINTING_HAND:
       name = "hand";
       break;
+    case CINNAMON_CURSOR_RESIZE_BOTTOM:
+      name = "bottom_side";
+      break;
+    case CINNAMON_CURSOR_RESIZE_TOP:
+      name = "top_side";
+      break;
+    case CINNAMON_CURSOR_RESIZE_LEFT:
+      name = "left_side";
+      break;
+    case CINNAMON_CURSOR_RESIZE_RIGHT:
+      name = "right_side";
+      break;
+    case CINNAMON_CURSOR_RESIZE_BOTTOM_RIGHT:
+      name = "bottom_right_corner";
+      break;
+    case CINNAMON_CURSOR_RESIZE_BOTTOM_LEFT:
+      name = "bottom_left_corner";
+      break;
+    case CINNAMON_CURSOR_RESIZE_TOP_RIGHT:
+      name = "top_right_corner";
+      break;
+    case CINNAMON_CURSOR_RESIZE_TOP_LEFT:
+      name = "top_left_corner";
+      break;
+    case CINNAMON_CURSOR_CROSSHAIR:
+      name = "crosshair";
+      break;
+    case CINNAMON_CURSOR_TEXT:
+      name = "xterm";
+      break;
     default:
       g_return_if_reached ();
     }
@@ -676,10 +671,40 @@ cinnamon_global_set_cursor (CinnamonGlobal *global,
         case CINNAMON_CURSOR_DND_UNSUPPORTED_TARGET:
           cursor_type = GDK_X_CURSOR;
           break;
+        case CINNAMON_CURSOR_RESIZE_BOTTOM:
+          cursor_type = GDK_BOTTOM_SIDE;
+          break;
+        case CINNAMON_CURSOR_RESIZE_TOP:
+          cursor_type = GDK_TOP_SIDE;
+          break;
+        case CINNAMON_CURSOR_RESIZE_LEFT:
+          cursor_type = GDK_LEFT_SIDE;
+          break;
+        case CINNAMON_CURSOR_RESIZE_RIGHT:
+          cursor_type = GDK_RIGHT_SIDE;
+          break;
+        case CINNAMON_CURSOR_RESIZE_BOTTOM_RIGHT:
+          cursor_type = GDK_BOTTOM_RIGHT_CORNER;
+          break;
+        case CINNAMON_CURSOR_RESIZE_BOTTOM_LEFT:
+          cursor_type = GDK_BOTTOM_LEFT_CORNER;
+          break;
+        case CINNAMON_CURSOR_RESIZE_TOP_RIGHT:
+          cursor_type = GDK_TOP_RIGHT_CORNER;
+          break;
+        case CINNAMON_CURSOR_RESIZE_TOP_LEFT:
+          cursor_type = GDK_TOP_LEFT_CORNER;
+          break;
+        case CINNAMON_CURSOR_CROSSHAIR:
+          cursor_type = GDK_CROSSHAIR;
+          break;
+        case CINNAMON_CURSOR_TEXT:
+          cursor_type = GDK_XTERM;
+          break;
         default:
           g_return_if_reached ();
         }
-      cursor = gdk_cursor_new (cursor_type);
+      cursor = gdk_cursor_new_for_display (gdk_display_get_default(), cursor_type);
     }
 
   gdk_window_set_cursor (global->stage_gdk_window, cursor);
@@ -824,60 +849,22 @@ global_stage_notify_height (GObject    *gobject,
   g_object_notify (G_OBJECT (global), "screen-height");
 }
 
-static void
-global_stage_before_paint (ClutterStage *stage,
-                           CinnamonGlobal  *global)
+static gboolean
+global_stage_before_paint (gpointer data)
 {
   cinnamon_perf_log_event (cinnamon_perf_log_get_default (),
                         "clutter.stagePaintStart");
+
+  return TRUE;
 }
 
-static void
-global_stage_after_paint (ClutterStage *stage,
-                          CinnamonGlobal  *global)
+static gboolean
+global_stage_after_paint (gpointer data)
 {
   cinnamon_perf_log_event (cinnamon_perf_log_get_default (),
                         "clutter.stagePaintDone");
-}
 
-static void
-constrain_tooltip (StTooltip             *tooltip,
-                   const ClutterGeometry *geometry,
-                   ClutterGeometry       *adjusted_geometry,
-                   gpointer               data)
-{
-  const ClutterGeometry *tip_area = st_tooltip_get_tip_area (tooltip);
-  CinnamonGlobal *global = cinnamon_global_get ();
-  MetaScreen *screen = cinnamon_global_get_screen (global);
-  int n_monitors = meta_screen_get_n_monitors (screen);
-  int i;
-
-  *adjusted_geometry = *geometry;
-
-  /* A point that determines what screen we'll constrain to */
-  int x = tip_area->x + tip_area->width / 2;
-  int y = tip_area->y + tip_area->height / 2;
-
-  for (i = 0; i < n_monitors; i++)
-    {
-      MetaRectangle rect;
-      meta_screen_get_monitor_geometry (screen, i, &rect);
-      if (x >= rect.x && x < rect.x + rect.width &&
-          y >= rect.y && y < rect.y + rect.height)
-        {
-          if (adjusted_geometry->x + adjusted_geometry->width > rect.x + rect.width)
-            adjusted_geometry->x = rect.x + rect.width - adjusted_geometry->width;
-          if (adjusted_geometry->x < rect.x)
-            adjusted_geometry->x = rect.x;
-
-          if (adjusted_geometry->y + adjusted_geometry->height > rect.y + rect.height)
-            adjusted_geometry->y = rect.y + rect.height - adjusted_geometry->height;
-          if (adjusted_geometry->y < rect.y)
-            adjusted_geometry->y = rect.y;
-
-          return;
-        }
-    }
+  return TRUE;
 }
 
 static void
@@ -992,15 +979,17 @@ update_scale_factor (GtkSettings *settings,
                      GParamSpec *pspec,
                      gpointer data)
 {
-  gint scale = 1;
+  guint scale = 1;
+  int xft_dpi;
+  GtkSettings *gtk_settings;
   CinnamonGlobal *global = CINNAMON_GLOBAL (data);
   ClutterStage *stage = CLUTTER_STAGE (global->stage);
   StThemeContext *context = st_theme_context_get_for_stage (stage);
   GValue value = G_VALUE_INIT;
 
-  g_value_init (&value, G_TYPE_INT);
+  g_value_init (&value, G_TYPE_UINT);
   if (gdk_screen_get_setting (global->gdk_screen, "gdk-window-scaling-factor", &value)) {
-    scale = g_value_get_int (&value);
+    scale = g_value_get_uint (&value);
     g_object_set (context, "scale-factor", scale, NULL);
 
     if (scale != global->ui_scale) {
@@ -1009,12 +998,14 @@ update_scale_factor (GtkSettings *settings,
     }
   }
 
-  g_settings_set_int (global->settings, "active-display-scale", scale);
+  meta_prefs_set_ui_scale (global->ui_scale);
 
-   /* Make sure clutter and gdk scaling stays disabled
-    * window-scaling-factor doesn't exist yet in clutter < 1.18, but it's a harmless warning
-    * and lets those that can, take advantage of it */
-  g_object_set (clutter_settings_get_default (), "window-scaling-factor", 1, NULL);
+  gtk_settings = gtk_settings_get_default ();
+
+  g_object_get (gtk_settings, "gtk-xft-dpi", &xft_dpi, NULL);
+  g_object_set (clutter_settings_get_default (), "font-dpi", xft_dpi, NULL);
+
+   /* Make sure gdk scaling stays disabled */
   gdk_x11_display_set_window_scale (gdk_display_get_default (), 1);
 }
 
@@ -1046,19 +1037,25 @@ _cinnamon_global_set_plugin (CinnamonGlobal *global,
   g_signal_connect (global->stage, "notify::height",
                     G_CALLBACK (global_stage_notify_height), global);
 
-  g_signal_connect (global->stage, "paint",
-                    G_CALLBACK (global_stage_before_paint), global);
-  g_signal_connect_after (global->stage, "paint",
-                          G_CALLBACK (global_stage_after_paint), global);
 
-  cinnamon_perf_log_define_event (cinnamon_perf_log_get_default(),
-                               "clutter.stagePaintStart",
-                               "Start of stage page repaint",
-                               "");
-  cinnamon_perf_log_define_event (cinnamon_perf_log_get_default(),
-                               "clutter.stagePaintDone",
-                               "End of stage page repaint",
-                               "");
+
+  if (g_getenv ("CINNAMON_PERF_OUTPUT") != NULL)
+    {
+      clutter_threads_add_repaint_func_full (CLUTTER_REPAINT_FLAGS_PRE_PAINT,
+                                             (GSourceFunc) global_stage_before_paint,
+                                             NULL, NULL);
+      clutter_threads_add_repaint_func_full (CLUTTER_REPAINT_FLAGS_POST_PAINT,
+                                             (GSourceFunc) global_stage_after_paint,
+                                             NULL, NULL);
+      cinnamon_perf_log_define_event (cinnamon_perf_log_get_default(),
+                                      "clutter.stagePaintStart",
+                                      "Start of stage page repaint",
+                                      "");
+      cinnamon_perf_log_define_event (cinnamon_perf_log_get_default(),
+                                      "clutter.stagePaintDone",
+                                      "End of stage page repaint",
+                                      "");
+    }
 
   g_signal_connect (global->meta_display, "notify::focus-window",
                     G_CALLBACK (focus_window_changed), global);
@@ -1075,6 +1072,18 @@ _cinnamon_global_set_plugin (CinnamonGlobal *global,
   update_scale_factor (gtk_settings_get_default (), NULL, global);
 }
 
+/**
+ * cinnamon_global_dump_gjs_stack:
+ * @global: A #CinnamonGlobal
+ *
+ * Prints out the gjs stack
+ */
+void
+cinnamon_global_dump_gjs_stack (CinnamonGlobal *global)
+{
+  gjs_dumpstack ();
+}
+
 GjsContext *
 _cinnamon_global_get_gjs_context (CinnamonGlobal *global)
 {
@@ -1091,7 +1100,7 @@ _cinnamon_global_get_gjs_context (CinnamonGlobal *global)
  * overview mode or the "looking glass" debug overlay, that block
  * application and normal key shortcuts.
  *
- * Returns value: %TRUE if we succesfully entered the mode. %FALSE if we couldn't
+ * Returns value: %TRUE if we successfully entered the mode. %FALSE if we couldn't
  *  enter the mode. Failure may occur because an application has the pointer
  *  or keyboard grabbed, because Muffin is in a mode itself like moving a
  *  window or alt-Tab window selection, or because cinnamon_global_begin_modal()
@@ -1099,10 +1108,11 @@ _cinnamon_global_get_gjs_context (CinnamonGlobal *global)
  */
 gboolean
 cinnamon_global_begin_modal (CinnamonGlobal *global,
-                          guint32      timestamp)
+                          guint32      timestamp,
+                          MetaModalOptions  options)
 {
   return meta_plugin_begin_modal (global->plugin, global->stage_xwindow,
-                                  None, 0, timestamp);
+                                  None, options, timestamp);
 }
 
 /**
@@ -1166,279 +1176,30 @@ cinnamon_global_destroy_pointer_barrier (CinnamonGlobal *global, guint32 barrier
 #endif
 }
 
-
-/**
- * cinnamon_global_add_extension_importer:
- * @target_object_script: JavaScript code evaluating to a target object
- * @target_property: Name of property to use for importer
- * @directory: Source directory:
- * @error: A #GError
- *
- * This function sets a property named @target_property on the object
- * resulting from the evaluation of @target_object_script code, which
- * acts as a GJS importer for directory @directory.
- *
- * Returns: %TRUE on success
- */
-gboolean
-cinnamon_global_add_extension_importer (CinnamonGlobal *global,
-                                     const char  *target_object_script,
-                                     const char  *target_property,
-                                     const char  *directory,
-                                     GError     **error)
-{
-  jsval target_object;
-  JSContext *context = gjs_context_get_native_context (global->js_context);
-  char *search_path[2] = { 0, 0 };
-
-  JS_BeginRequest (context);
-
-  // This is a bit of a hack; ideally we'd be able to pass our target
-  // object directly into this function, but introspection doesn't
-  // support that at the moment.  Instead evaluate a string to get it.
-  if (!JS_EvaluateScript(context,
-                         JS_GetGlobalObject(context),
-                         target_object_script,
-                         strlen (target_object_script),
-                         "<target_object_script>",
-                         0,
-                         &target_object))
-    {
-      char *message;
-      gjs_log_exception(context,
-                        &message);
-      g_set_error(error,
-                  G_IO_ERROR,
-                  G_IO_ERROR_FAILED,
-                  "%s", message ? message : "(unknown)");
-      g_free(message);
-      goto out_error;
-    }
-
-  if (!JSVAL_IS_OBJECT (target_object))
-    {
-      g_error ("cinnamon_global_add_extension_importer: invalid target object");
-      goto out_error;
-    }
-
-  search_path[0] = (char*)directory;
-  gjs_define_importer (context, JSVAL_TO_OBJECT (target_object), target_property, (const char **)search_path, FALSE);
-  JS_EndRequest (context);
-  return TRUE;
- out_error:
-  JS_EndRequest (context);
-  return FALSE;
-}
-
-/* Code to close all file descriptors before we exec; copied from gspawn.c in GLib.
- *
- * Authors: Padraig O'Briain, Matthias Clasen, Lennart Poettering
- *
- * http://bugzilla.gnome.org/show_bug.cgi?id=469231
- * http://bugzilla.gnome.org/show_bug.cgi?id=357585
- */
-
-static int
-set_cloexec (void *data, gint fd)
-{
-  if (fd >= GPOINTER_TO_INT (data))
-    fcntl (fd, F_SETFD, FD_CLOEXEC);
-
-  return 0;
-}
-
-#ifndef HAVE_FDWALK
-static int
-fdwalk (int (*cb)(void *data, int fd), void *data)
-{
-  gint open_max;
-  gint fd;
-  gint res = 0;
-
-#ifdef HAVE_SYS_RESOURCE_H
-  struct rlimit rl;
-#endif
-
-#ifdef __linux__
-  DIR *d;
-
-  if ((d = opendir("/proc/self/fd"))) {
-      struct dirent *de;
-
-      while ((de = readdir(d))) {
-          glong l;
-          gchar *e = NULL;
-
-          if (de->d_name[0] == '.')
-              continue;
-
-          errno = 0;
-          l = strtol(de->d_name, &e, 10);
-          if (errno != 0 || !e || *e)
-              continue;
-
-          fd = (gint) l;
-
-          if ((glong) fd != l)
-              continue;
-
-          if (fd == dirfd(d))
-              continue;
-
-          if ((res = cb (data, fd)) != 0)
-              break;
-        }
-
-      closedir(d);
-      return res;
-  }
-
-  /* If /proc is not mounted or not accessible we fall back to the old
-   * rlimit trick */
-
-#endif
-
-#ifdef HAVE_SYS_RESOURCE_H
-  if (getrlimit(RLIMIT_NOFILE, &rl) == 0 && rl.rlim_max != RLIM_INFINITY)
-      open_max = rl.rlim_max;
-  else
-#endif
-      open_max = sysconf (_SC_OPEN_MAX);
-
-  for (fd = 0; fd < open_max; fd++)
-      if ((res = cb (data, fd)) != 0)
-          break;
-
-  return res;
-}
-#endif
-
-static void
-pre_exec_close_fds(void)
-{
-  fdwalk (set_cloexec, GINT_TO_POINTER(3));
-}
-
 /**
  * cinnamon_global_reexec_self:
  * @global: A #CinnamonGlobal
- * 
- * Restart the current process.  Only intended for development purposes. 
+ *
+ * Restart the current process.  Only intended for development purposes.
  */
-void 
+void
 cinnamon_global_reexec_self (CinnamonGlobal *global)
 {
-  GPtrArray *arr;
-  gsize len;
-  char *buf;
-  char *buf_p;
-  char *buf_end;
-  GError *error = NULL;
-  
-  /* Linux specific (I think, anyways). */
-  if (!g_file_get_contents ("/proc/self/cmdline", &buf, &len, &error))
-    {
-      g_warning ("failed to get /proc/self/cmdline: %s", error->message);
-      return;
-    }
-      
-  buf_end = buf+len;
-  arr = g_ptr_array_new ();
-  /* The cmdline file is NUL-separated */
-  for (buf_p = buf; buf_p < buf_end; buf_p = buf_p + strlen (buf_p) + 1)
-    g_ptr_array_add (arr, buf_p);
-  
-  g_ptr_array_add (arr, NULL);
-
-  /* Close all file descriptors other than stdin/stdout/stderr, otherwise
-   * they will leak and stay open after the exec. In particular, this is
-   * important for file descriptors that represent mapped graphics buffer
-   * objects.
-   */
-  pre_exec_close_fds ();
-
-  meta_display_unmanage_screen (cinnamon_global_get_display (global),
-                                cinnamon_global_get_screen (global),
-                                cinnamon_global_get_current_time (global));
-
-  execvp (arr->pdata[0], (char**)arr->pdata);
-  g_warning ("failed to reexec: %s", g_strerror (errno));
-  g_ptr_array_free (arr, TRUE);
+  meta_restart ();
 }
 
-/**
- * cinnamon_global_gc:
- * @global: A #CinnamonGlobal
- *
- * Start a garbage collection process.  For more information, see
- * https://developer.mozilla.org/En/JS_GC
- */
 void
-cinnamon_global_gc (CinnamonGlobal *global)
+cinnamon_global_shutdown (void)
 {
-  JSContext *context = gjs_context_get_native_context (global->js_context);
+    g_signal_emit_by_name (the_object, "shutdown");
 
-  JS_GC (context);
+    meta_pre_exec_close_fds ();
+
+    meta_display_unmanage_screen (cinnamon_global_get_display (the_object),
+                                  cinnamon_global_get_screen (the_object),
+                                  cinnamon_global_get_current_time (the_object));
 }
 
-/**
- * cinnamon_global_maybe_gc:
- * @global: A #CinnamonGlobal
- *
- * Start a garbage collection process when it would free up enough memory
- * to be worth the amount of time it would take
- * https://developer.mozilla.org/en/SpiderMonkey/JSAPI_Reference/JS_MaybeGC
- */
-void
-cinnamon_global_maybe_gc (CinnamonGlobal *global)
-{
-  gjs_context_maybe_gc (global->js_context);
-}
-
-static void
-cinnamon_global_on_gc (GjsContext   *context,
-                    CinnamonGlobal  *global)
-{
-  global->last_gc_end_time = g_get_monotonic_time ();
-}
-
-/**
- * cinnamon_global_get_memory_info:
- * @global:
- * @meminfo: (out caller-allocates): Output location for memory information
- *
- * Load process-global data about memory usage.
- */
-void
-cinnamon_global_get_memory_info (CinnamonGlobal        *global,
-                              CinnamonMemoryInfo    *meminfo)
-{
-  JSContext *context;
-  gint64 now;
-
-  memset (meminfo, 0, sizeof (*meminfo));
-#ifdef HAVE_MALLINFO
-  {
-    struct mallinfo info = mallinfo ();
-    meminfo->glibc_uordblks = info.uordblks;
-  }
-#endif
-
-  context = gjs_context_get_native_context (global->js_context);
-
-  meminfo->js_bytes = JS_GetGCParameter (JS_GetRuntime (context), JSGC_BYTES);
-
-  meminfo->gjs_boxed = (unsigned int) gjs_counter_boxed.value;
-  meminfo->gjs_gobject = (unsigned int) gjs_counter_object.value;
-  meminfo->gjs_function = (unsigned int) gjs_counter_function.value;
-  meminfo->gjs_closure = (unsigned int) gjs_counter_closure.value;
-
-  now = g_get_monotonic_time ();
-
-  meminfo->last_gc_seconds_ago = (now - global->last_gc_end_time) / G_TIME_SPAN_SECOND;
-}
-
-
 /**
  * cinnamon_global_notify_error:
  * @global: a #CinnamonGlobal
@@ -1462,7 +1223,7 @@ static void
 grab_notify (GtkWidget *widget, gboolean was_grabbed, gpointer user_data)
 {
   CinnamonGlobal *global = CINNAMON_GLOBAL (user_data);
-  
+
   global->gtk_grab_active = !was_grabbed;
 
   /* Update for the new setting of gtk_grab_active */
@@ -1519,7 +1280,7 @@ cinnamon_global_get_pointer (CinnamonGlobal         *global,
   GdkDevice *gdevice;
   GdkScreen *gscreen;
   GdkModifierType raw_mods;
-  
+
   gmanager = gdk_display_get_device_manager (global->gdk_display);
   gdevice = gdk_device_manager_get_client_pointer (gmanager);
   gdk_device_get_position (gdevice, &gscreen, x, y);
@@ -1548,7 +1309,7 @@ cinnamon_global_set_pointer (CinnamonGlobal         *global,
   GdkDevice *gdevice;
   GdkScreen *gscreen;
   int x2, y2;
-  
+
   gmanager = gdk_display_get_device_manager (global->gdk_display);
   gdevice = gdk_device_manager_get_client_pointer (gmanager);
   gdk_device_get_position (gdevice, &gscreen, &x2, &y2);
@@ -1661,22 +1422,22 @@ cinnamon_global_get_current_time (CinnamonGlobal *global)
 /**
  * cinnamon_global_get_pid:
  *
- * Returns: the pid of the cinnamon process.
+ * Return value: the pid of the cinnamon process.
  */
 pid_t
-cinnamon_global_get_pid ()
+cinnamon_global_get_pid (CinnamonGlobal *global)
 {
   return getpid();
 }
 
 /**
  * cinnamon_global_get_md5_for_string:
+ * @string: input string
  *
- * Returns: the MD5 sum for the given string
+ * Return value: (transfer full): the MD5 sum for the given string
  */
-
 gchar *
-cinnamon_global_get_md5_for_string (const gchar *string)
+cinnamon_global_get_md5_for_string (CinnamonGlobal *global, const gchar *string)
 {
     return g_compute_checksum_for_string (G_CHECKSUM_MD5, string, -1);
 }
@@ -1696,7 +1457,7 @@ cinnamon_global_create_app_launch_context (CinnamonGlobal *global)
   GdkAppLaunchContext *context;
 
   context = gdk_display_get_app_launch_context (global->gdk_display);
-  
+
   gdk_app_launch_context_set_timestamp (context, cinnamon_global_get_current_time (global));
 
   // Make sure that the app is opened on the current workspace even if
@@ -1726,13 +1487,6 @@ run_leisure_functions (gpointer data)
   if (global->work_count > 0)
     return FALSE;
 
-  /* Previously we called gjs_maybe_gc().  However, it simply doesn't
-   * trigger often enough.  Garbage collection is very fast here, so
-   * let's just aggressively GC.  This will help avoid both heap
-   * fragmentation, and the GC kicking in when we don't want it to.
-   */
-  gjs_context_gc (global->js_context);
-
   /* No leisure closures, so we are done */
   if (global->leisure_closures == NULL)
     return FALSE;
@@ -1848,54 +1602,6 @@ cinnamon_global_run_at_leisure (CinnamonGlobal         *global,
     schedule_leisure_functions (global);
 }
 
-/**
- * cinnamon_global_play_theme_sound:
- * @global: the #CinnamonGlobal
- * @id: an id, used to cancel later (0 if not needed)
- * @name: the sound name
- *
- * Plays a simple sound picked according to Freedesktop sound theme.
- * Really just a workaround for libcanberra not being introspected.
- */
-void
-cinnamon_global_play_theme_sound (CinnamonGlobal *global,
-                               guint        id,
-                               const char  *name)
-{
-  ca_context_play (global->sound_context, id, CA_PROP_EVENT_ID, name, NULL);
-}
-
-/**
- * cinnamon_global_play_sound_file:
- * @global: the #CinnamonGlobal
- * @id: an id, used to cancel later (0 if not needed)
- * @filename: the filename of the sound
- *
- * Plays a simple sound.
- * Really just a workaround for libcanberra not being introspected.
- */
-void
-cinnamon_global_play_sound_file (CinnamonGlobal *global,
-                               guint        id,
-                               const char  *filename)
-{
-  ca_context_play (global->sound_context, id, CA_PROP_MEDIA_FILENAME, filename, NULL);
-}
-
-/**
- * cinnamon_global_cancel_sound:
- * @global: the #CinnamonGlobal
- * @id: the id previously passed to cinnamon_global_play_theme_sound() or cinnamon_global_play_sound_file()
- *
- * Cancels a sound being played.
- */
-void
-cinnamon_global_cancel_sound (CinnamonGlobal *global,
-                                 guint id)
-{
-  ca_context_cancel (global->sound_context, id);
-}
-
 /*
  * Process Xdnd events
  *
@@ -1953,3 +1659,46 @@ gboolean _cinnamon_global_check_xdnd_event (CinnamonGlobal  *global,
 
     return FALSE;
 }
+
+/**
+ * cinnamon_global_segfault:
+ * @global: the #CinnamonGlobal
+ *
+ * Crashes Cinnamon by causing a segfault
+ */
+void
+cinnamon_global_segfault (CinnamonGlobal *global)
+{
+  int *ptr = NULL;
+  g_strdup_printf ("%d", *ptr);
+}
+
+/**
+ * cinnamon_global_alloc_leak:
+ * @global: the #CinnamonGlobal
+ * @mb: How many mb to leak
+ *
+ * Request mb megabytes allocated. This is just for debugging.
+ */
+void
+cinnamon_global_alloc_leak (CinnamonGlobal *global, gint mb)
+{
+    gint i;
+    gchar x;
+
+    for (i = 0; i < mb * 1024; i++)
+    {
+        gchar *ptr = g_strdup_printf ("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
+                                      "xxxxxxxxxxxxxxxxxxxxxxxx"
+        );
+    }
+}
\ No newline at end of file
