@@ -36,12 +36,14 @@ MetaDisplay   *cinnamon_global_get_display               (CinnamonGlobal *global
 GList         *cinnamon_global_get_window_actors         (CinnamonGlobal *global);
 GSettings     *cinnamon_global_get_settings              (CinnamonGlobal *global);
 guint32        cinnamon_global_get_current_time          (CinnamonGlobal *global);
-pid_t          cinnamon_global_get_pid                      (void);
-gchar         *cinnamon_global_get_md5_for_string        (const gchar *string);
+pid_t          cinnamon_global_get_pid                   (CinnamonGlobal *global);
+gchar         *cinnamon_global_get_md5_for_string        (CinnamonGlobal *global, const gchar *string);
+void           cinnamon_global_dump_gjs_stack            (CinnamonGlobal *global);
 
 /* Input/event handling */
 gboolean cinnamon_global_begin_modal            (CinnamonGlobal         *global,
-                                              guint32              timestamp);
+                                              guint32              timestamp,
+                                              MetaModalOptions    options);
 void     cinnamon_global_end_modal              (CinnamonGlobal         *global,
                                               guint32              timestamp);
 
@@ -63,7 +65,17 @@ typedef enum {
   CINNAMON_CURSOR_DND_UNSUPPORTED_TARGET,
   CINNAMON_CURSOR_DND_MOVE,
   CINNAMON_CURSOR_DND_COPY,
-  CINNAMON_CURSOR_POINTING_HAND
+  CINNAMON_CURSOR_POINTING_HAND,
+  CINNAMON_CURSOR_RESIZE_BOTTOM,
+  CINNAMON_CURSOR_RESIZE_TOP,
+  CINNAMON_CURSOR_RESIZE_LEFT,
+  CINNAMON_CURSOR_RESIZE_RIGHT,
+  CINNAMON_CURSOR_RESIZE_BOTTOM_RIGHT,
+  CINNAMON_CURSOR_RESIZE_BOTTOM_LEFT,
+  CINNAMON_CURSOR_RESIZE_TOP_RIGHT,
+  CINNAMON_CURSOR_RESIZE_TOP_LEFT,
+  CINNAMON_CURSOR_CROSSHAIR,
+  CINNAMON_CURSOR_TEXT
 } CinnamonCursor;
 
 void    cinnamon_global_set_cursor              (CinnamonGlobal         *global,
@@ -87,29 +99,6 @@ void    cinnamon_global_set_pointer             (CinnamonGlobal         *global,
                                               int                 x,
                                               int                 y);
 
-
-/* JavaScript utilities */
-void     cinnamon_global_gc                   (CinnamonGlobal *global);
-void     cinnamon_global_maybe_gc             (CinnamonGlobal *global);
-
-typedef struct {
-  guint glibc_uordblks;
-
-  guint js_bytes;
-
-  guint gjs_boxed;
-  guint gjs_gobject;
-  guint gjs_function;
-  guint gjs_closure;
-
-  /* 32 bit to avoid js conversion problems with 64 bit */
-  guint  last_gc_seconds_ago;
-} CinnamonMemoryInfo;
-
-void     cinnamon_global_get_memory_info      (CinnamonGlobal     *global,
-                                            CinnamonMemoryInfo *meminfo);
-
-
 /* Run-at-leisure API */
 void cinnamon_global_begin_work     (CinnamonGlobal          *global);
 void cinnamon_global_end_work       (CinnamonGlobal          *global);
@@ -123,36 +112,25 @@ void cinnamon_global_run_at_leisure (CinnamonGlobal          *global,
 
 
 /* Misc utilities / Cinnamon API */
-gboolean cinnamon_global_add_extension_importer    (CinnamonGlobal  *global,
-                                                 const char   *target_object_script,
-                                                 const char   *target_property,
-                                                 const char   *directory,
-                                                 GError      **error);
 
 void     cinnamon_global_sync_pointer              (CinnamonGlobal  *global);
 
 GAppLaunchContext *
          cinnamon_global_create_app_launch_context (CinnamonGlobal  *global);
 
-void     cinnamon_global_play_theme_sound          (CinnamonGlobal  *global,
-                                                 guint         id,
-                                                 const char   *name);
-
-void     cinnamon_global_play_sound_file          (CinnamonGlobal  *global,
-                                                 guint         id,
-                                                 const char   *filename);
-
-void     cinnamon_global_cancel_sound             (CinnamonGlobal  *global,
-                                                 guint         id);
-
 void     cinnamon_global_notify_error              (CinnamonGlobal  *global,
                                                  const char   *msg,
                                                  const char   *details);
 
 void     cinnamon_global_init_xdnd                 (CinnamonGlobal  *global);
 
+void     cinnamon_global_shutdown                  (void);
 void     cinnamon_global_reexec_self               (CinnamonGlobal  *global);
 
+void     cinnamon_global_segfault                  (CinnamonGlobal  *global);
+void     cinnamon_global_alloc_leak                (CinnamonGlobal  *global,
+                                                    gint             mb);
+
 G_END_DECLS
 
 #endif /* __CINNAMON_GLOBAL_H__ */
