@@ -37,15 +37,10 @@
 
 #include "st-scroll-bar.h"
 #include "st-bin.h"
-#include "st-marshal.h"
 #include "st-enum-types.h"
 #include "st-private.h"
 #include "st-button.h"
 
-G_DEFINE_TYPE (StScrollBar, st_scroll_bar, ST_TYPE_WIDGET)
-
-#define ST_SCROLL_BAR_GET_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), ST_TYPE_SCROLL_BAR, StScrollBarPrivate))
-
 #define PAGING_INITIAL_REPEAT_TIMEOUT 500
 #define PAGING_SUBSEQUENT_REPEAT_TIMEOUT 200
 
@@ -57,8 +52,6 @@ struct _StScrollBarPrivate
   gfloat        x_origin;
   gfloat        y_origin;
 
-  ClutterActor *bw_stepper;
-  ClutterActor *fw_stepper;
   ClutterActor *trough;
   ClutterActor *handle;
 
@@ -70,14 +63,13 @@ struct _StScrollBarPrivate
   guint             paging_source_id;
   guint             paging_event_no;
 
-  gboolean          stepper_forward;
-  guint             stepper_source_id;
-
   ClutterAnimation *paging_animation;
 
   guint             vertical : 1;
 };
 
+G_DEFINE_TYPE_WITH_PRIVATE (StScrollBar, st_scroll_bar, ST_TYPE_WIDGET)
+
 enum
 {
   PROP_0,
@@ -146,23 +138,11 @@ st_scroll_bar_set_property (GObject      *gobject,
     case PROP_VERTICAL:
       bar->priv->vertical = g_value_get_boolean (value);
       if (bar->priv->vertical)
-        {
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->bw_stepper),
-                                  "up-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->fw_stepper),
-                                  "down-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->handle),
-                                  "vhandle");
-        }
+        clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->handle),
+                                "vhandle");
       else
-        {
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->fw_stepper),
-                                  "forward-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->bw_stepper),
-                                  "backward-stepper");
-          clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->handle),
-                                  "hhandle");
-        }
+        clutter_actor_set_name (CLUTTER_ACTOR (bar->priv->handle),
+                                "hhandle");
       clutter_actor_queue_relayout ((ClutterActor*) gobject);
       break;
 
@@ -183,67 +163,19 @@ st_scroll_bar_dispose (GObject *gobject)
 
   if (priv->handle)
     {
-      g_signal_handlers_disconnect_by_func (priv->handle,
-                                            G_CALLBACK (handle_button_press_event_cb),
-                                            bar);
-      clutter_actor_unparent (priv->handle);
+      clutter_actor_destroy (priv->handle);
       priv->handle = NULL;
     }
 
-  if (priv->bw_stepper)
-    {
-      clutter_actor_unparent (priv->bw_stepper);
-      priv->bw_stepper = NULL;
-    }
-
-  if (priv->fw_stepper)
-    {
-      clutter_actor_unparent (priv->fw_stepper);
-      priv->fw_stepper = NULL;
-    }
-
   if (priv->trough)
     {
-      clutter_actor_unparent (priv->trough);
+      clutter_actor_destroy (priv->trough);
       priv->trough = NULL;
     }
 
   G_OBJECT_CLASS (st_scroll_bar_parent_class)->dispose (gobject);
 }
 
-static void
-st_scroll_bar_paint (ClutterActor *actor)
-{
-  StScrollBarPrivate *priv = ST_SCROLL_BAR (actor)->priv;
-
-  CLUTTER_ACTOR_CLASS (st_scroll_bar_parent_class)->paint (actor);
-
-  clutter_actor_paint (priv->bw_stepper);
-
-  clutter_actor_paint (priv->fw_stepper);
-
-  clutter_actor_paint (priv->trough);
-
-  if (priv->handle && CLUTTER_ACTOR_IS_VISIBLE (priv->handle))
-    clutter_actor_paint (priv->handle);
-}
-
-static void
-st_scroll_bar_pick (ClutterActor       *actor,
-                    const ClutterColor *pick_color)
-{
-  StScrollBarPrivate *priv = ST_SCROLL_BAR (actor)->priv;
-
-  CLUTTER_ACTOR_CLASS (st_scroll_bar_parent_class)->pick (actor, pick_color);
-
-  clutter_actor_paint (priv->bw_stepper);
-  clutter_actor_paint (priv->fw_stepper);
-  clutter_actor_paint (priv->trough);
-
-  if (priv->handle && priv->adjustment)
-    clutter_actor_paint (priv->handle);
-}
-
 static void
 st_scroll_bar_unmap (ClutterActor *actor)
 {
@@ -259,90 +191,34 @@ scroll_bar_allocate_children (StScrollBar           *bar,
 {
   StScrollBarPrivate *priv = bar->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (bar));
-  ClutterActorBox content_box, bw_box, fw_box, trough_box;
-  gfloat bw_stepper_size, fw_stepper_size, min_size, natural_size;
+  ClutterActorBox content_box, trough_box;
 
   st_theme_node_get_content_box (theme_node, box, &content_box);
 
   if (priv->vertical)
     {
-      gfloat width = content_box.x2 - content_box.x1;
-
-      clutter_actor_get_preferred_height (priv->bw_stepper, width,
-                                          &min_size, &natural_size);
-      bw_stepper_size = MAX (min_size, natural_size);
-
-      /* Backward stepper */
-      bw_box.x1 = content_box.x1;
-      bw_box.y1 = content_box.y1;
-      bw_box.x2 = content_box.x2;
-      bw_box.y2 = bw_box.y1 + bw_stepper_size;
-      clutter_actor_allocate (priv->bw_stepper, &bw_box, flags);
-
-
-      clutter_actor_get_preferred_height (priv->fw_stepper, width,
-                                          &min_size, &natural_size);
-      fw_stepper_size = MAX (min_size, natural_size);
-
-      /* Forward stepper */
-      fw_box.x1 = content_box.x1;
-      fw_box.y1 = content_box.y2 - fw_stepper_size;
-      fw_box.x2 = content_box.x2;
-      fw_box.y2 = content_box.y2;
-      clutter_actor_allocate (priv->fw_stepper, &fw_box, flags);
-
-      /* Trough */
       trough_box.x1 = content_box.x1;
-      trough_box.y1 = content_box.y1 + bw_stepper_size;
+      trough_box.y1 = content_box.y1;
       trough_box.x2 = content_box.x2;
-      trough_box.y2 = content_box.y2 - fw_stepper_size;
+      trough_box.y2 = content_box.y2;
       clutter_actor_allocate (priv->trough, &trough_box, flags);
 
     }
   else
     {
-      gfloat height = content_box.y2 - content_box.y1;
-
-      clutter_actor_get_preferred_width (priv->bw_stepper, height,
-                                         &min_size, &natural_size);
-      bw_stepper_size = MAX (min_size, natural_size);
-
-      /* Backward stepper */
-      bw_box.x1 = content_box.x1;
-      bw_box.y1 = content_box.y1;
-      bw_box.x2 = bw_box.x1 + bw_stepper_size;
-      bw_box.y2 = content_box.y2;
-      clutter_actor_allocate (priv->bw_stepper, &bw_box, flags);
-
-
-      clutter_actor_get_preferred_width (priv->fw_stepper, height,
-                                         &min_size, &natural_size);
-      fw_stepper_size = MAX (min_size, natural_size);
-
-      /* Forward stepper */
-      fw_box.x1 = content_box.x2 - fw_stepper_size;
-      fw_box.y1 = content_box.y1;
-      fw_box.x2 = content_box.x2;
-      fw_box.y2 = content_box.y2;
-      clutter_actor_allocate (priv->fw_stepper, &fw_box, flags);
-
-      /* Trough */
-      trough_box.x1 = content_box.x1 + bw_stepper_size;
+      trough_box.x1 = content_box.x1;
       trough_box.y1 = content_box.y1;
-      trough_box.x2 = content_box.x2 - fw_stepper_size;
+      trough_box.x2 = content_box.x2;
       trough_box.y2 = content_box.y2;
       clutter_actor_allocate (priv->trough, &trough_box, flags);
     }
 
-
   if (priv->adjustment)
     {
-      float handle_size, position, avail_size, stepper_size;
+      float handle_size, position, avail_size;
       gdouble value, lower, upper, page_size, increment, min_size, max_size;
       ClutterActorBox handle_box = { 0, };
 
-      stepper_size = bw_stepper_size + fw_stepper_size;
-
       st_adjustment_get_values (priv->adjustment,
                                 &value,
                                 &lower,
@@ -369,35 +245,29 @@ scroll_bar_allocate_children (StScrollBar           *bar,
 
       if (priv->vertical)
         {
-          avail_size = content_box.y2 - content_box.y1 - stepper_size;
+          avail_size = content_box.y2 - content_box.y1;
           handle_size = increment * avail_size;
           handle_size = CLAMP (handle_size, min_size, max_size);
 
           handle_box.x1 = content_box.x1;
-          handle_box.y1 = bw_box.y2 + position * (avail_size - handle_size);
+          handle_box.y1 = content_box.y1 + floor (position * (avail_size - handle_size));
 
           handle_box.x2 = content_box.x2;
-          handle_box.y2 = handle_box.y1 + handle_size;
+          handle_box.y2 = handle_box.y1 + floor (handle_size);
         }
       else
         {
-          avail_size = content_box.x2 - content_box.x1 - stepper_size;
+          avail_size = content_box.x2 - content_box.x1;
           handle_size = increment * avail_size;
           handle_size = CLAMP (handle_size, min_size, max_size);
 
-          handle_box.x1 = bw_box.x2 + position * (avail_size - handle_size);
+          handle_box.x1 = content_box.x1 + floor (position * (avail_size - handle_size));
           handle_box.y1 = content_box.y1;
 
-          handle_box.x2 = handle_box.x1 + handle_size;
+          handle_box.x2 = handle_box.x1 + floor (handle_size);
           handle_box.y2 = content_box.y2;
         }
 
-      /* snap to pixel */
-      handle_box.x1 = (int) handle_box.x1;
-      handle_box.y1 = (int) handle_box.y1;
-      handle_box.x2 = (int) handle_box.x2;
-      handle_box.y2 = (int) handle_box.y2;
-
       clutter_actor_allocate (priv->handle,
                               &handle_box,
                               flags);
@@ -413,51 +283,32 @@ st_scroll_bar_get_preferred_width (ClutterActor *self,
   StScrollBar *bar = ST_SCROLL_BAR (self);
   StScrollBarPrivate *priv = bar->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));
+  gfloat trough_min_width, trough_natural_width;
+  gfloat handle_min_width, handle_natural_width;
 
   st_theme_node_adjust_for_height (theme_node, &for_height);
 
-  if (min_width_p)
-    *min_width_p = 0;
+  _st_actor_get_preferred_width (priv->trough, for_height, TRUE,
+                                 &trough_min_width, &trough_natural_width);
+
+  _st_actor_get_preferred_width (priv->handle, for_height, TRUE,
+                                 &handle_min_width, &handle_natural_width);
 
-  if (natural_width_p)
-    *natural_width_p = 0;
   if (priv->vertical)
     {
-      gfloat tmin_width_p, tnatural_width_p;
+      if (min_width_p)
+        *min_width_p = MAX (trough_min_width, handle_min_width);
 
-      #define ADJUST_WIDTH_IF_LARGER(actor) \
-        _st_actor_get_preferred_width (actor, for_height, TRUE, \
-                                       &tmin_width_p, &tnatural_width_p); \
-        if (min_width_p && tmin_width_p > *min_width_p) \
-          *min_width_p = tmin_width_p; \
-        if (natural_width_p && tnatural_width_p > *natural_width_p) \
-          *natural_width_p = tnatural_width_p;
-
-      ADJUST_WIDTH_IF_LARGER (priv->bw_stepper);
-      ADJUST_WIDTH_IF_LARGER (priv->fw_stepper);
-      ADJUST_WIDTH_IF_LARGER (priv->trough);
-      ADJUST_WIDTH_IF_LARGER (priv->handle);
-
-      #undef ADJUST_WIDTH_IF_LARGER
+      if (natural_width_p)
+        *natural_width_p = MAX (trough_natural_width, handle_natural_width);
     }
   else
     {
-      gfloat tmin_width_p, tnatural_width_p;
+      if (min_width_p)
+        *min_width_p = trough_min_width + handle_min_width;
 
-      #define ADD_TO_WIDTH(actor) \
-        _st_actor_get_preferred_width (actor, for_height, TRUE, \
-                                       &tmin_width_p, &tnatural_width_p); \
-        if (min_width_p) \
-          *min_width_p += tmin_width_p; \
-        if (natural_width_p ) \
-          *natural_width_p += tnatural_width_p;
-
-      ADD_TO_WIDTH (priv->bw_stepper);
-      ADD_TO_WIDTH (priv->fw_stepper);
-      ADD_TO_WIDTH (priv->trough);
-      ADD_TO_WIDTH (priv->handle);
-
-      #undef ADD_TO_WIDTH
+      if (natural_width_p)
+        *natural_width_p = trough_natural_width + handle_natural_width;
     }
 
   st_theme_node_adjust_preferred_width (theme_node, min_width_p, natural_width_p);
@@ -472,51 +323,32 @@ st_scroll_bar_get_preferred_height (ClutterActor *self,
   StScrollBar *bar = ST_SCROLL_BAR (self);
   StScrollBarPrivate *priv = bar->priv;
   StThemeNode *theme_node = st_widget_get_theme_node (ST_WIDGET (self));
+  gfloat trough_min_height, trough_natural_height;
+  gfloat handle_min_height, handle_natural_height;
 
   st_theme_node_adjust_for_width (theme_node, &for_width);
 
-  if (min_height_p)
-    *min_height_p = 0;
+  _st_actor_get_preferred_height (priv->trough, for_width, TRUE,
+                                  &trough_min_height, &trough_natural_height);
+
+  _st_actor_get_preferred_height (priv->handle, for_width, TRUE,
+                                  &handle_min_height, &handle_natural_height);
 
-  if (natural_height_p)
-    *natural_height_p = 0;
   if (priv->vertical)
     {
-      gfloat tmin_height_p, tnatural_height_p;
+      if (min_height_p)
+        *min_height_p = trough_min_height + handle_min_height;
 
-      #define ADD_TO_HEIGHT(actor) \
-        _st_actor_get_preferred_height (actor, for_width, FALSE, \
-                                        &tmin_height_p, &tnatural_height_p); \
-        if (min_height_p) \
-          *min_height_p += tmin_height_p; \
-        if (natural_height_p) \
-          *natural_height_p += tnatural_height_p;
-
-      ADD_TO_HEIGHT (priv->bw_stepper);
-      ADD_TO_HEIGHT (priv->fw_stepper);
-      ADD_TO_HEIGHT (priv->trough);
-      ADD_TO_HEIGHT (priv->handle);
-
-      #undef ADD_TO_HEIGHT
+      if (natural_height_p)
+        *natural_height_p = trough_natural_height + handle_natural_height;
     }
   else
     {
-      gfloat tmin_height_p, tnatural_height_p;
+      if (min_height_p)
+        *min_height_p = MAX (trough_min_height, handle_min_height);
 
-      #define ADJUST_HEIGHT_IF_LARGER(actor) \
-        _st_actor_get_preferred_height (actor, for_width, FALSE, \
-                                        &tmin_height_p, &tnatural_height_p); \
-        if (min_height_p && tmin_height_p > *min_height_p) \
-          *min_height_p = tmin_height_p; \
-        if (natural_height_p && tnatural_height_p > *natural_height_p) \
-          *natural_height_p = tnatural_height_p;
-
-      ADJUST_HEIGHT_IF_LARGER (priv->bw_stepper);
-      ADJUST_HEIGHT_IF_LARGER (priv->fw_stepper);
-      ADJUST_HEIGHT_IF_LARGER (priv->trough);
-      ADJUST_HEIGHT_IF_LARGER (priv->handle);
-
-      #undef ADJUST_HEIGHT_IF_LARGER
+      if (natural_height_p)
+        *natural_height_p = MAX (trough_natural_height, handle_natural_height);
     }
 
   st_theme_node_adjust_preferred_height (theme_node, min_height_p, natural_height_p);
@@ -529,8 +361,7 @@ st_scroll_bar_allocate (ClutterActor          *actor,
 {
   StScrollBar *bar = ST_SCROLL_BAR (actor);
 
-  /* Chain up */
-  CLUTTER_ACTOR_CLASS (st_scroll_bar_parent_class)->allocate (actor, box, flags);
+  clutter_actor_set_allocation (actor, box, flags);
 
   scroll_bar_allocate_children (bar, box, flags);
 }
@@ -563,8 +394,6 @@ st_scroll_bar_style_changed (StWidget *widget)
 {
   StScrollBarPrivate *priv = ST_SCROLL_BAR (widget)->priv;
 
-  st_widget_style_changed (ST_WIDGET (priv->bw_stepper));
-  st_widget_style_changed (ST_WIDGET (priv->fw_stepper));
   st_widget_style_changed (ST_WIDGET (priv->trough));
   st_widget_style_changed (ST_WIDGET (priv->handle));
 
@@ -639,8 +468,13 @@ st_scroll_bar_scroll_event (ClutterActor       *actor,
     case CLUTTER_SCROLL_RIGHT:
       st_adjustment_set_value (priv->adjustment, value + step);
       break;
+    default:
+          g_return_val_if_reached (FALSE);
+      break;
     }
 
+  clutter_actor_queue_redraw (actor);
+
   return TRUE;
 }
 
@@ -652,8 +486,6 @@ st_scroll_bar_class_init (StScrollBarClass *klass)
   StWidgetClass *widget_class = ST_WIDGET_CLASS (klass);
   GParamSpec *pspec;
 
-  g_type_class_add_private (klass, sizeof (StScrollBarPrivate));
-
   object_class->get_property = st_scroll_bar_get_property;
   object_class->set_property = st_scroll_bar_set_property;
   object_class->dispose      = st_scroll_bar_dispose;
@@ -662,8 +494,6 @@ st_scroll_bar_class_init (StScrollBarClass *klass)
   actor_class->get_preferred_width  = st_scroll_bar_get_preferred_width;
   actor_class->get_preferred_height = st_scroll_bar_get_preferred_height;
   actor_class->allocate       = st_scroll_bar_allocate;
-  actor_class->paint          = st_scroll_bar_paint;
-  actor_class->pick           = st_scroll_bar_pick;
   actor_class->scroll_event   = st_scroll_bar_scroll_event;
   actor_class->unmap          = st_scroll_bar_unmap;
 
@@ -690,8 +520,7 @@ st_scroll_bar_class_init (StScrollBarClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (StScrollBarClass, scroll_start),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   signals[SCROLL_STOP] =
@@ -699,8 +528,7 @@ st_scroll_bar_class_init (StScrollBarClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (StScrollBarClass, scroll_stop),
-                  NULL, NULL,
-                  g_cclosure_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 }
 
@@ -748,6 +576,8 @@ move_slider (StScrollBar *bar,
              + lower;
 
   st_adjustment_set_value (priv->adjustment, position);
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (bar));
 }
 
 static void
@@ -806,11 +636,13 @@ handle_button_press_event_cb (ClutterActor       *actor,
                               StScrollBar        *bar)
 {
   ClutterStage *stage;
-  StScrollBarPrivate *priv = bar->priv;
+  StScrollBarPrivate *priv;
 
   if (event->button != 1)
     return FALSE;
 
+  priv = bar->priv;
+
   if (!clutter_actor_transform_stage_point (priv->handle,
                                             event->x,
                                             event->y,
@@ -839,10 +671,17 @@ handle_button_press_event_cb (ClutterActor       *actor,
 
 static void
 animation_completed_cb (ClutterAnimation   *animation,
-                        StScrollBarPrivate *priv)
+                        StScrollBar        *self)
 {
-  g_object_unref (priv->paging_animation);
-  priv->paging_animation = NULL;
+  ClutterTimeline *t;
+
+  t = clutter_animation_get_timeline (animation);
+  g_signal_handlers_disconnect_by_func (t, clutter_actor_queue_redraw, self);
+
+  clutter_actor_queue_redraw (CLUTTER_ACTOR (self));
+
+  g_object_unref (self->priv->paging_animation);
+  self->priv->paging_animation = NULL;
 }
 
 static gboolean
@@ -954,7 +793,8 @@ trough_paging_cb (StScrollBar *self)
   clutter_animation_bind (self->priv->paging_animation, "value", &v);
   t = clutter_animation_get_timeline (self->priv->paging_animation);
   g_signal_connect (a, "completed", G_CALLBACK (animation_completed_cb),
-                    self->priv);
+                    self);
+  g_signal_connect_swapped (t, "new-frame", G_CALLBACK (clutter_actor_queue_redraw), self);
   clutter_timeline_start (t);
 
   return ret;
@@ -1014,121 +854,13 @@ trough_leave_event_cb (ClutterActor *actor,
   return FALSE;
 }
 
-static void
-stepper_animation_completed_cb (ClutterAnimation *a,
-                                gpointer          data)
-{
-  g_object_unref (a);
-}
-
-static void
-stepper_move_on (StScrollBarPrivate *priv,
-                 gint                mode)
-{
-  ClutterAnimation *a;
-  ClutterTimeline *t;
-  GValue v = { 0, };
-  double value, inc;
-
-  a = g_object_new (CLUTTER_TYPE_ANIMATION,
-                    "object", priv->adjustment,
-                    "duration", (guint)(PAGING_SUBSEQUENT_REPEAT_TIMEOUT * st_slow_down_factor),
-                    "mode", mode,
-                    NULL);
-
-  g_signal_connect (a, "completed", G_CALLBACK (stepper_animation_completed_cb),
-                    NULL);
-
-  g_object_get (priv->adjustment,
-                "step-increment", &inc,
-                "value", &value,
-                NULL);
-
-  if (priv->stepper_forward)
-    value = value + inc;
-  else
-    value = value - inc;
-
-  g_value_init (&v, G_TYPE_DOUBLE);
-  g_value_set_double (&v, value);
-  clutter_animation_bind (a, "value", &v);
-
-  t = clutter_animation_get_timeline (a);
-  clutter_timeline_start (t);
-}
-
-static gboolean
-stepper_button_subsequent_timeout (StScrollBarPrivate *priv)
-{
-  stepper_move_on (priv, CLUTTER_LINEAR);
-
-  return TRUE;
-}
-
-static gboolean
-stepper_button_repeat_timeout (StScrollBarPrivate *priv)
-{
-  priv->stepper_source_id = 0;
-
-  stepper_move_on (priv, CLUTTER_EASE_IN_CUBIC);
-
-  priv->stepper_source_id = g_timeout_add (PAGING_SUBSEQUENT_REPEAT_TIMEOUT,
-                                           (GSourceFunc)
-                                           stepper_button_subsequent_timeout,
-                                           priv);
-  return FALSE;
-}
-
-static gboolean
-stepper_button_press_event_cb (ClutterActor       *actor,
-                               ClutterButtonEvent *event,
-                               StScrollBar        *bar)
-{
-  StScrollBarPrivate *priv = bar->priv;
-
-  if (event->button != 1)
-    return FALSE;
-
-  if (bar->priv->adjustment == NULL)
-    return FALSE;
-
-  bar->priv->stepper_forward = (actor == priv->fw_stepper);
-
-  stepper_move_on (priv, CLUTTER_EASE_OUT_CUBIC);
-
-  priv->stepper_source_id = g_timeout_add (PAGING_INITIAL_REPEAT_TIMEOUT,
-                                           (GSourceFunc)
-                                           stepper_button_repeat_timeout,
-                                           priv);
-
-  return TRUE;
-}
-
-static gboolean
-stepper_button_release_cb (ClutterActor       *actor,
-                           ClutterButtonEvent *event,
-                           StScrollBar        *self)
-{
-  if (event->button != 1)
-    return FALSE;
-
-  if (self->priv->stepper_source_id) {
-    g_source_remove (self->priv->stepper_source_id);
-    self->priv->stepper_source_id = 0;
-  }
-
-  return FALSE;
-}
-
 static void
 st_scroll_bar_notify_reactive (StScrollBar *self)
 {
   StScrollBarPrivate *priv = self->priv;
 
-  gboolean reactive = CLUTTER_ACTOR_IS_REACTIVE (self);
+  gboolean reactive = clutter_actor_get_reactive (CLUTTER_ACTOR (self));
 
-  clutter_actor_set_reactive (CLUTTER_ACTOR (priv->bw_stepper), reactive);
-  clutter_actor_set_reactive (CLUTTER_ACTOR (priv->fw_stepper), reactive);
   clutter_actor_set_reactive (CLUTTER_ACTOR (priv->trough), reactive);
   clutter_actor_set_reactive (CLUTTER_ACTOR (priv->handle), reactive);
 }
@@ -1136,33 +868,13 @@ st_scroll_bar_notify_reactive (StScrollBar *self)
 static void
 st_scroll_bar_init (StScrollBar *self)
 {
-  self->priv = ST_SCROLL_BAR_GET_PRIVATE (self);
-
-  self->priv->bw_stepper = (ClutterActor *) st_button_new ();
-  clutter_actor_set_name (CLUTTER_ACTOR (self->priv->bw_stepper),
-                          "backward-stepper");
-  clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->bw_stepper),
-                            CLUTTER_ACTOR (self));
-  g_signal_connect (self->priv->bw_stepper, "button-press-event",
-                    G_CALLBACK (stepper_button_press_event_cb), self);
-  g_signal_connect (self->priv->bw_stepper, "button-release-event",
-                    G_CALLBACK (stepper_button_release_cb), self);
-
-  self->priv->fw_stepper = (ClutterActor *) st_button_new ();
-  clutter_actor_set_name (CLUTTER_ACTOR (self->priv->fw_stepper),
-                          "forward-stepper");
-  clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->fw_stepper),
-                            CLUTTER_ACTOR (self));
-  g_signal_connect (self->priv->fw_stepper, "button-press-event",
-                    G_CALLBACK (stepper_button_press_event_cb), self);
-  g_signal_connect (self->priv->fw_stepper, "button-release-event",
-                    G_CALLBACK (stepper_button_release_cb), self);
+  self->priv = st_scroll_bar_get_instance_private (self);
 
   self->priv->trough = (ClutterActor *) st_bin_new ();
   clutter_actor_set_reactive ((ClutterActor *) self->priv->trough, TRUE);
   clutter_actor_set_name (CLUTTER_ACTOR (self->priv->trough), "trough");
-  clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->trough),
-                            CLUTTER_ACTOR (self));
+  clutter_actor_add_child (CLUTTER_ACTOR (self),
+                           CLUTTER_ACTOR (self->priv->trough));
   g_signal_connect (self->priv->trough, "button-press-event",
                     G_CALLBACK (trough_button_press_event_cb), self);
   g_signal_connect (self->priv->trough, "button-release-event",
@@ -1172,8 +884,8 @@ st_scroll_bar_init (StScrollBar *self)
 
   self->priv->handle = (ClutterActor *) st_button_new ();
   clutter_actor_set_name (CLUTTER_ACTOR (self->priv->handle), "hhandle");
-  clutter_actor_set_parent (CLUTTER_ACTOR (self->priv->handle),
-                            self->priv->trough);
+  clutter_actor_add_child (CLUTTER_ACTOR (self),
+                           CLUTTER_ACTOR (self->priv->handle));
   g_signal_connect (self->priv->handle, "button-press-event",
                     G_CALLBACK (handle_button_press_event_cb), self);
 
@@ -1264,4 +976,3 @@ st_scroll_bar_get_adjustment (StScrollBar *bar)
 
   return bar->priv->adjustment;
 }
-
