@@ -31,8 +31,8 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <clutter/clutter.h>
 
-#include <st/st-types.h>
-#include <st/st-theme-node.h>
+#include "st-types.h"
+#include "st-theme-node.h"
 
 #define ST_TYPE_TEXTURE_CACHE                 (st_texture_cache_get_type ())
 #define ST_TEXTURE_CACHE(obj)                 (G_TYPE_CHECK_INSTANCE_CAST ((obj), ST_TYPE_TEXTURE_CACHE, StTextureCache))
@@ -121,6 +121,25 @@ ClutterActor *st_texture_cache_load_from_raw (StTextureCache    *cache,
 ClutterActor *st_texture_cache_load_file_simple (StTextureCache *cache,
                                                  const gchar    *file_path);
 
+/**
+ * StTextureCacheLoadImageCallback
+ * @cache: a #StTextureCache
+ * @actor: the actor containing the loaded image
+ * @gpointer: Callback data
+ *
+ * Callback from st_texture_cache_load_image_from_file_async
+ */
+typedef void (* StTextureCacheLoadImageCallback) (StTextureCache *cache,
+                                                  ClutterActor   *actor,
+                                                  gpointer        user_data);
+
+void st_texture_cache_load_image_from_file_async (StTextureCache                    *cache,
+                                                  const gchar                       *path,
+                                                  gint                               width,
+                                                  gint                               height,
+                                                  StTextureCacheLoadImageCallback    callback,
+                                                  gpointer                           user_data);
+
 /**
  * StTextureCacheLoader: (skip)
  * @cache: a #StTextureCache
